@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -33,8 +34,8 @@
  * @ingroup epx
  */
 
-#ifndef RELIC_EPX_H
-#define RELIC_EPX_H
+#ifndef RLC_EPX_H
+#define RLC_EPX_H
 
 #include "relic_fpx.h"
 #include "relic_ep.h"
@@ -47,59 +48,50 @@
 /**
  * Size of a precomputation table using the binary method.
  */
-#define RELIC_EPX_TABLE_BASIC		(2 * FP_BITS + 1)
-
-/**
- * Size of a precomputation table using Yao's windowing method.
- */
-#define RELIC_EPX_TABLE_YAOWI      (2 * FP_BITS / EP_DEPTH + 1)
-
-/**
- * Size of a precomputation table using the NAF windowing method.
- */
-#define RELIC_EPX_TABLE_NAFWI      (2 * FP_BITS / EP_DEPTH + 1)
+#define RLC_EPX_TABLE_BASIC		(2 * RLC_FP_BITS + 1)
 
 /**
  * Size of a precomputation table using the single-table comb method.
  */
-#define RELIC_EPX_TABLE_COMBS      (1 << EP_DEPTH)
+#define RLC_EPX_TABLE_COMBS      (1 << EP_DEPTH)
 
 /**
  * Size of a precomputation table using the double-table comb method.
  */
-#define RELIC_EPX_TABLE_COMBD		(1 << (EP_DEPTH + 1))
+#define RLC_EPX_TABLE_COMBD		(1 << (EP_DEPTH + 1))
 
 /**
  * Size of a precomputation table using the w-(T)NAF method.
  */
-#define RELIC_EPX_TABLE_LWNAF		(1 << (EP_DEPTH - 2))
+#define RLC_EPX_TABLE_LWNAF		(1 << (EP_DEPTH - 2))
 
 /**
  * Size of a precomputation table using the chosen algorithm.
  */
 #if EP_FIX == BASIC
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_BASIC
-#elif EP_FIX == YAOWI
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_YAOWI
-#elif EP_FIX == NAFWI
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_NAFWI
+#define RLC_EPX_TABLE			RLC_EPX_TABLE_BASIC
 #elif EP_FIX == COMBS
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_COMBS
+#define RLC_EPX_TABLE			RLC_EPX_TABLE_COMBS
 #elif EP_FIX == COMBD
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_COMBD
+#define RLC_EPX_TABLE			RLC_EPX_TABLE_COMBD
 #elif EP_FIX == LWNAF
-#define RELIC_EPX_TABLE			RELIC_EPX_TABLE_LWNAF
+#define RLC_EPX_TABLE			RLC_EPX_TABLE_LWNAF
 #endif
 
 /**
  * Maximum size of a precomputation table.
  */
 #ifdef STRIP
-#define RELIC_EPX_TABLE_MAX RELIC_EPX_TABLE
+#define RLC_EPX_TABLE_MAX 	RLC_EPX_TABLE
 #else
-#define RELIC_EPX_TABLE_MAX MAX(RELIC_EPX_TABLE_BASIC, RELIC_EPX_TABLE_COMBD)
+#define RLC_EPX_TABLE_MAX 	RLC_MAX(RLC_EPX_TABLE_BASIC, RLC_EPX_TABLE_COMBD)
 #endif
 
+/**
+ * Maximum number of coefficients of an isogeny map polynomial.
+ * 4 is sufficient for a degree-3 isogeny polynomial.
+ */
+#define RLC_EPX_CTMAP_MAX	4
 
 /*============================================================================*/
 /* Type definitions                                                           */
@@ -153,6 +145,42 @@ typedef ep3_st ep3_t[1];
 typedef ep3_st *ep3_t;
 #endif
 
+/**
+ * Coefficients of an isogeny map for a curve over a quadratic extension.
+ */
+typedef struct {
+	/** The a-coefficient of the isogenous curve used for SSWU mapping. */
+	fp2_t a;
+	/** The b-coefficient of the isogenous curve used for SSWU mapping. */
+	fp2_t b;
+	/** Degree of x numerator */
+	int deg_xn;
+	/** Degree of x denominator */
+	int deg_xd;
+	/** Degree of y numerator */
+	int deg_yn;
+	/** Degree of y denominator */
+	int deg_yd;
+	/** x numerator coefficients */
+	fp2_t xn[RLC_EPX_CTMAP_MAX];
+	/** x denominator coefficients */
+	fp2_t xd[RLC_EPX_CTMAP_MAX];
+	/** y numerator coefficients */
+	fp2_t yn[RLC_EPX_CTMAP_MAX];
+	/** y denominator coefficients */
+	fp2_t yd[RLC_EPX_CTMAP_MAX];
+#if ALLOC == STACK
+	/** In case of stack allocation, storage for the values in this struct. */
+	/* a, b, and the elms in xn, xd, yn, yd */
+	fp2_st storage[2 + 4 * RLC_EPX_CTMAP_MAX];
+#endif /* ALLOC == DYNAMIC or STACK */
+} iso2_st;
+
+/**
+ * Pointer to isogeny map coefficients.
+ */
+typedef iso2_st *iso2_t;
+
 /*============================================================================*/
 /* Macro definitions                                                          */
 /*============================================================================*/
@@ -187,19 +215,6 @@ typedef ep3_st *ep3_t;
 	fp2_new((A)->y);														\
 	fp2_new((A)->z);														\
 
-#elif ALLOC == STATIC
-#define ep2_new(A)															\
-	A = (ep2_t)alloca(sizeof(ep2_st));										\
-	if (A == NULL) {														\
-		THROW(ERR_NO_MEMORY);												\
-	}																		\
-	fp2_null((A)->x);														\
-	fp2_null((A)->y);														\
-	fp2_null((A)->z);														\
-	fp2_new((A)->x);														\
-	fp2_new((A)->y);														\
-	fp2_new((A)->z);														\
-
 #elif ALLOC == AUTO
 #define ep2_new(A)				/* empty */
 
@@ -227,15 +242,6 @@ typedef ep3_st *ep3_t;
 		A = NULL;															\
 	}																		\
 
-#elif ALLOC == STATIC
-#define ep2_free(A)															\
-	if (A != NULL) {														\
-		fp2_free((A)->x);													\
-		fp2_free((A)->y);													\
-		fp2_free((A)->z);													\
-		A = NULL;															\
-	}																		\
-
 #elif ALLOC == AUTO
 #define ep2_free(A)				/* empty */
 #elif ALLOC == STACK
@@ -310,7 +316,7 @@ typedef ep3_st *ep3_t;
 #define ep2_mul(R, P, K)		ep2_mul_slide(R, P, K)
 #elif EP_MUL == MONTY
 #define ep2_mul(R, P, K)		ep2_mul_monty(R, P, K)
-#elif EP_MUL == LWNAF
+#elif EP_MUL == LWNAF || EP2_MUL == LWREG
 #define ep2_mul(R, P, K)		ep2_mul_lwnaf(R, P, K)
 #endif
 
@@ -323,10 +329,6 @@ typedef ep3_st *ep3_t;
  */
 #if EP_FIX == BASIC
 #define ep2_mul_pre(T, P)		ep2_mul_pre_basic(T, P)
-#elif EP_FIX == YAOWI
-#define ep2_mul_pre(T, P)		ep2_mul_pre_yaowi(T, P)
-#elif EP_FIX == NAFWI
-#define ep2_mul_pre(T, P)		ep2_mul_pre_nafwi(T, P)
 #elif EP_FIX == COMBS
 #define ep2_mul_pre(T, P)		ep2_mul_pre_combs(T, P)
 #elif EP_FIX == COMBD
@@ -348,10 +350,6 @@ typedef ep3_st *ep3_t;
  */
 #if EP_FIX == BASIC
 #define ep2_mul_fix(R, T, K)	ep2_mul_fix_basic(R, T, K)
-#elif EP_FIX == YAOWI
-#define ep2_mul_fix(R, T, K)	ep2_mul_fix_yaowi(R, T, K)
-#elif EP_FIX == NAFWI
-#define ep2_mul_fix(R, T, K)	ep2_mul_fix_nafwi(R, T, K)
 #elif EP_FIX == COMBS
 #define ep2_mul_fix(R, T, K)	ep2_mul_fix_combs(R, T, K)
 #elif EP_FIX == COMBD
@@ -400,16 +398,16 @@ void ep2_curve_clean(void);
 /**
  * Returns the 'a' coefficient of the currently configured elliptic curve.
  *
- * @param[out] a			- the 'a' coefficient of the elliptic curve.
+ * @return the 'a' coefficient of the elliptic curve.
  */
-void ep2_curve_get_a(fp2_t a);
+fp_t *ep2_curve_get_a(void);
 
 /**
  * Returns the 'b' coefficient of the currently configured elliptic curve.
  *
  * @param[out] b			- the 'b' coefficient of the elliptic curve.
  */
-void ep2_curve_get_b(fp2_t b);
+fp_t *ep2_curve_get_b(void);
 
 /**
  * Returns the vector of coefficients required to perform GLV method.
@@ -425,6 +423,13 @@ void ep2_curve_get_vs(bn_t *v);
  */
 int ep2_curve_opt_a(void);
 
+/**
+ * Returns b optimization identifier based on the 'b' coefficient of the curve.
+ *
+ * @return the optimization identifier.
+ */
+int ep2_curve_opt_b(void);
+
 /**
  * Tests if the configured elliptic curve is a twist.
  *
@@ -432,6 +437,13 @@ int ep2_curve_opt_a(void);
  */
 int ep2_curve_is_twist(void);
 
+/**
+ * Tests if the current curve should use an isogeny map for the SSWU map.
+ *
+ * @return 1 if the curve uses an isogeny, and 0 otherwise.
+ */
+int ep2_curve_is_ctmap(void);
+
 /**
  * Returns the generator of the group of points in the elliptic curve.
  *
@@ -474,6 +486,11 @@ void ep2_curve_get_s3(bn_t s3);
  */
 void ep2_curve_get_s32(bn_t s32);
 
+/**
+ * Returns the isogeny map coefficients for use with the SSWU map.
+ */
+iso2_t ep2_curve_get_iso(void);
+
 /**
  * Configures an elliptic curve over a quadratic extension by its coefficients.
  *
@@ -520,7 +537,7 @@ void ep2_copy(ep2_t r, ep2_t p);
  *
  * @param[in] p				- the first elliptic curve point.
  * @param[in] q				- the second elliptic curve point.
- * @return CMP_EQ if p == q and CMP_NE if p != q.
+ * @return RLC_EQ if p == q and RLC_NE if p != q.
  */
 int ep2_cmp(ep2_t p, ep2_t q);
 
@@ -583,7 +600,7 @@ int ep2_size_bin(ep2_t a, int pack);
  * @throw ERR_NO_VALID		- if the encoded point is invalid.
  * @throw ERR_NO_BUFFER		- if the buffer capacity is invalid.
  */
-void ep2_read_bin(ep2_t a, uint8_t *bin, int len);
+void ep2_read_bin(ep2_t a, const uint8_t *bin, int len);
 
 /**
  * Writes a prime elliptic curve pointer over a quadratic extension to a byte
@@ -749,6 +766,15 @@ void ep2_mul_lwreg(ep2_t r, ep2_t p, const bn_t k);
  */
 void ep2_mul_gen(ep2_t r, bn_t k);
 
+/**
+ * Multiplies a prime elliptic point by a small integer.
+ *
+ * @param[out] r			- the result.
+ * @param[in] p				- the point to multiply.
+ * @param[in] k				- the integer.
+ */
+void ep2_mul_dig(ep2_t r, ep2_t p, dig_t k);
+
 /**
  * Builds a precomputation table for multiplying a fixed prime elliptic point
  * using the binary method.
@@ -923,13 +949,15 @@ void ep2_mul_sim_joint(ep2_t r, ep2_t p, bn_t k, ep2_t q, bn_t m);
 void ep2_mul_sim_gen(ep2_t r, bn_t k, ep2_t q, bn_t m);
 
 /**
- * Multiplies a prime elliptic point by a small integer.
+ * Multiplies prime elliptic curve points by small scalars.
+ * Computes R = \sum k_iP_i.
  *
  * @param[out] r			- the result.
- * @param[in] p				- the point to multiply.
- * @param[in] k				- the integer.
+ * @param[in] p				- the points to multiply.
+ * @param[in] k				- the small scalars.
+ * @param[in] len			- the number of points to multiply.
  */
-void ep2_mul_dig(ep2_t r, ep2_t p, dig_t k);
+void ep2_mul_sim_dig(ep2_t r, ep2_t p[], dig_t k[], int len);
 
 /**
  * Converts a point to affine coordinates.
@@ -949,12 +977,14 @@ void ep2_norm(ep2_t r, ep2_t p);
 void ep2_norm_sim(ep2_t *r, ep2_t *t, int n);
 
 /**
- * Maps a byte array to a point in an elliptic curve over a quadratic extension.
+ * Maps a byte array to a point in a prime elliptic curve. The
+ * algorithm implemented is the Fouque-Tibouchi algorithm from the
+ * paper "Indifferentiable Hashing to Barreto-Naehrig curves" for
+ * the BLS12-381 curve.
  *
  * @param[out] p			- the result.
  * @param[in] msg			- the byte array to map.
  * @param[in] len			- the array length in bytes.
- * @param[in] performHash	- whether to hash internally
  */
 void ep2_map(ep2_t p, const uint8_t *msg, int len, int performHash);
 
@@ -989,4 +1019,4 @@ void ep2_pck(ep2_t r, ep2_t p);
  */
 int ep2_upk(ep2_t r, ep2_t p);
 
-#endif /* !RELIC_EPX_H */
+#endif /* !RLC_EPX_H */
