@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -32,14 +33,13 @@
  * @ingroup fb
  */
 
-#ifndef RELIC_FB_H
-#define RELIC_FB_H
+#ifndef RLC_FB_H
+#define RLC_FB_H
 
 #include "relic_bn.h"
 #include "relic_dv.h"
 #include "relic_conf.h"
 #include "relic_types.h"
-#include "relic_label.h"
 
 /*============================================================================*/
 /* Constant definitions                                                       */
@@ -48,27 +48,17 @@
 /**
  * Precision in bits of a binary field element.
  */
-#define FB_BITS 	((int)FB_POLYN)
-
-/**
- * Size in bits of a digit.
- */
-#define FB_DIGIT	((int)DIGIT)
-
-/**
- * Logarithm of the digit size in base 2.
- */
-#define FB_DIG_LOG	((int)DIGIT_LOG)
+#define RLC_FB_BITS 	((int)FB_POLYN)
 
 /**
  * Size in digits of a block sufficient to store a binary field element.
  */
-#define FB_DIGS		((int)((FB_BITS)/(FB_DIGIT) + (FB_BITS % FB_DIGIT > 0)))
+#define RLC_FB_DIGS		((int)RLC_CEIL(RLC_FB_BITS, RLC_DIG))
 
 /**
  * Size in bytes of a block sufficient to store a binary field element.
  */
-#define FB_BYTES 	((int)((FB_BITS)/8 + (FB_BITS % 8 > 0)))
+#define RLC_FB_BYTES 	((int)RLC_CEIL(RLC_FB_BITS, 8))
 
 /**
  * Finite field identifiers.
@@ -134,31 +124,31 @@ enum {
  * Size of a precomputation table for repeated squaring/square-root using the
  * trivial approach.
  */
-#define RELIC_FB_TABLE_BASIC		(1)
+#define RLC_FB_TABLE_BASIC		(1)
 
 /**
  * Size of a precomputation table for repeated squaring/square-root using the
  * faster approach.
  */
-#define RELIC_FB_TABLE_QUICK      ((FB_DIGIT / 4) * FB_DIGS * 16)
+#define RLC_FB_TABLE_QUICK      ((RLC_DIG / 4) * RLC_FB_DIGS * 16)
 
 /**
  * Size of a precomputation table for repeated squaring/square-root using the
  * chosen algorithm.
  */
 #if FB_ITR == BASIC
-#define RELIC_FB_TABLE 			RELIC_FB_TABLE_BASIC
+#define RLC_FB_TABLE 			RLC_FB_TABLE_BASIC
 #else
-#define RELIC_FB_TABLE			RELIC_FB_TABLE_QUICK
+#define RLC_FB_TABLE			RLC_FB_TABLE_QUICK
 #endif
 
 /**
  * Maximum size of a precomputation table.
  */
 #ifdef STRIP
-#define RELIC_FB_TABLE_MAX 		RELIC_FB_TABLE
+#define RLC_FB_TABLE_MAX 		RLC_FB_TABLE
 #else
-#define RELIC_FB_TABLE_MAX 		RELIC_FB_TABLE_QUICK
+#define RLC_FB_TABLE_MAX 		RLC_FB_TABLE_QUICK
 #endif
 
 /*============================================================================*/
@@ -169,7 +159,7 @@ enum {
  * Represents a binary field element.
  */
 #if ALLOC == AUTO
-typedef relic_align dig_t fb_t[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
+typedef rlc_align dig_t fb_t[RLC_FB_DIGS + RLC_PAD(RLC_FB_BYTES) / (RLC_DIG / 8)];
 #else
 typedef dig_t *fb_t;
 #endif
@@ -177,7 +167,7 @@ typedef dig_t *fb_t;
 /**
  * Represents a binary field element with automatic memory allocation.
  */
-typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
+typedef rlc_align dig_t fb_st[RLC_FB_DIGS + RLC_PAD(RLC_FB_BYTES) / (RLC_DIG / 8)];
 
 /*============================================================================*/
 /* Macro definitions                                                          */
@@ -189,7 +179,7 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[out] A			- the binary field element to initialize.
  */
 #if ALLOC == AUTO
-#define fb_null(A)			/* empty */
+#define fb_null(A)				/* empty */
 #else
 #define fb_null(A)			A = NULL;
 #endif
@@ -201,15 +191,13 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @throw ERR_NO_MEMORY		- if there is no available memory.
  */
 #if ALLOC == DYNAMIC
-#define fb_new(A)			dv_new_dynam((dv_t *)&(A), FB_DIGS)
-#elif ALLOC == STATIC
-#define fb_new(A)			dv_new_statc((dv_t *)&(A), FB_DIGS)
+#define fb_new(A)			dv_new_dynam((dv_t *)&(A), RLC_FB_DIGS)
 #elif ALLOC == AUTO
-#define fb_new(A)			/* empty */
+#define fb_new(A)				/* empty */
 #elif ALLOC == STACK
 #define fb_new(A)															\
-	A = (dig_t *)alloca(FB_BYTES + PADDING(FB_BYTES));						\
-	A = (dig_t *)ALIGNED(A);												\
+	A = (dig_t *)alloca(RLC_FB_BYTES + RLC_PAD(RLC_FB_BYTES));						\
+	A = (dig_t *)RLC_ALIGN(A);												\
 
 #endif
 
@@ -220,10 +208,8 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  */
 #if ALLOC == DYNAMIC
 #define fb_free(A)			dv_free_dynam((dv_t *)&(A))
-#elif ALLOC == STATIC
-#define fb_free(A)			dv_free_statc((dv_t *)&(A))
 #elif ALLOC == AUTO
-#define fb_free(A)			/* empty */
+#define fb_free(A)				/* empty */
 #elif ALLOC == STACK
 #define fb_free(A)			A = NULL;
 #endif
@@ -236,17 +222,13 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] B				- the second binary field element to multiply.
  */
 #if FB_KARAT > 0
-#define fb_mul(C, A, B)	fb_mul_karat(C, A, B)
+#define fb_mul(C, A, B)		fb_mul_karat(C, A, B)
 #elif FB_MUL == BASIC
-#define fb_mul(C, A, B)	fb_mul_basic(C, A, B)
+#define fb_mul(C, A, B)		fb_mul_basic(C, A, B)
 #elif FB_MUL == INTEG
-#define fb_mul(C, A, B)	fb_mul_integ(C, A, B)
-#elif FB_MUL == LCOMB
-#define fb_mul(C, A, B)	fb_mul_lcomb(C, A, B)
-#elif FB_MUL == RCOMB
-#define fb_mul(C, A, B)	fb_mul_rcomb(C, A, B)
+#define fb_mul(C, A, B)		fb_mul_integ(C, A, B)
 #elif FB_MUL == LODAH
-#define fb_mul(C, A, B)	fb_mul_lodah(C, A, B)
+#define fb_mul(C, A, B)		fb_mul_lodah(C, A, B)
 #endif
 
 /**
@@ -256,11 +238,11 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] A				- the binary field element to square.
  */
 #if FB_SQR == BASIC
-#define fb_sqr(C, A)	fb_sqr_basic(C, A)
-#elif FB_SQR == LUTBL
-#define fb_sqr(C, A)	fb_sqr_lutbl(C, A)
+#define fb_sqr(C, A)		fb_sqr_basic(C, A)
+#elif FB_SQR == QUICK
+#define fb_sqr(C, A)		fb_sqr_quick(C, A)
 #elif FB_SQR == INTEG
-#define fb_sqr(C, A)	fb_sqr_integ(C, A)
+#define fb_sqr(C, A)		fb_sqr_integ(C, A)
 #endif
 
 /**
@@ -270,9 +252,9 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] A				- the binary field element.
  */
 #if FB_SRT == BASIC
-#define fb_srt(C, A)	fb_srt_basic(C, A)
+#define fb_srt(C, A)		fb_srt_basic(C, A)
 #elif FB_SRT == QUICK
-#define fb_srt(C, A)	fb_srt_quick(C, A)
+#define fb_srt(C, A)		fb_srt_quick(C, A)
 #endif
 
 /**
@@ -283,9 +265,9 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] A				- the multiplication result to reduce.
  */
 #if FB_RDC == BASIC
-#define fb_rdc(C, A)	fb_rdc_basic(C, A)
+#define fb_rdc(C, A)		fb_rdc_basic(C, A)
 #elif FB_RDC == QUICK
-#define fb_rdc(C, A)	fb_rdc_quick(C, A)
+#define fb_rdc(C, A)		fb_rdc_quick(C, A)
 #endif
 
 /**
@@ -295,9 +277,9 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @return the trace of the binary field element.
  */
 #if FB_TRC == BASIC
-#define fb_trc(A)		fb_trc_basic(A)
+#define fb_trc(A)			fb_trc_basic(A)
 #elif FB_TRC == QUICK
-#define fb_trc(A)		fb_trc_quick(A)
+#define fb_trc(A)			fb_trc_quick(A)
 #endif
 
 /**
@@ -308,9 +290,9 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] A				- the binary field element.
  */
 #if FB_SLV == BASIC
-#define fb_slv(C, A)	fb_slv_basic(C, A)
+#define fb_slv(C, A)		fb_slv_basic(C, A)
 #elif FB_SLV == QUICK
-#define fb_slv(C, A)	fb_slv_quick(C, A)
+#define fb_slv(C, A)		fb_slv_quick(C, A)
 #endif
 
 /**
@@ -320,17 +302,21 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] A				- the binary field element to invert.
  */
 #if FB_INV == BASIC
-#define fb_inv(C, A)	fb_inv_basic(C, A)
+#define fb_inv(C, A)		fb_inv_basic(C, A)
 #elif FB_INV == BINAR
-#define fb_inv(C, A)	fb_inv_binar(C, A)
-#elif FB_INV == LOWER
-#define fb_inv(C, A)	fb_inv_lower(C, A)
+#define fb_inv(C, A)		fb_inv_binar(C, A)
 #elif FB_INV == EXGCD
-#define fb_inv(C, A)	fb_inv_exgcd(C, A)
+#define fb_inv(C, A)		fb_inv_exgcd(C, A)
 #elif FB_INV == ALMOS
-#define fb_inv(C, A)	fb_inv_almos(C, A)
+#define fb_inv(C, A)		fb_inv_almos(C, A)
 #elif FB_INV == ITOHT
-#define fb_inv(C, A)	fb_inv_itoht(C, A)
+#define fb_inv(C, A)		fb_inv_itoht(C, A)
+#elif FB_INV == BRUCH
+#define fb_inv(C, A)		fb_inv_bruch(C, A)
+#elif FB_INV == CTAIA
+#define fb_inv(C, A)		fb_inv_ctaia(C, A)
+#elif FB_INV == LOWER
+#define fb_inv(C, A)		fb_inv_lower(C, A)
 #endif
 
 /**
@@ -374,27 +360,25 @@ typedef relic_align dig_t fb_st[FB_DIGS + PADDING(FB_BYTES)/(FB_DIGIT / 8)];
  * @param[in] B				- the exponent.
  * @param[in] ...			- the modulus and an optional argument.
  */
-#define fb_itr(C, A, ...)CAT(fb_itr, OPT(__VA_ARGS__))(C, A, __VA_ARGS__)
+#define fb_itr(C, A, ...)	RLC_CAT(fb_itr, RLC_OPT(__VA_ARGS__)) (C, A, __VA_ARGS__)
 
 /**
  * Reduces a multiple precision integer modulo another integer. This macro
  * should not be called directly. Use bn_mod() with 4 arguments instead.
  *
- * @param[out] C			- the result.
- * @param[in] A				- the binary field element to exponentiate.
- * @param[in] B				- the exponent.
- * @param[in] T				- the precomputed table for the exponent.
+ * @param[out] C				- the result.
+ * @param[in] A					- the binary field element to exponentiate.
+ * @param[in] B					- the exponent.
+ * @param[in] T					- the precomputed table for the exponent.
  */
 #if FB_ITR == BASIC
 #define fb_itr_imp(C, A, B, T)		fb_itr_basic(C, A, B)
 #elif FB_ITR == QUICK
 #define fb_itr_imp(C, A, B, T)		fb_itr_quick(C, A, T)
 #endif
-
 /*============================================================================*/
-/* Function prototypes                                                        */
+		/* Function prototypes                                                        */
 /*============================================================================*/
-
 /**
  * Initializes the binary field arithmetic layer.
  */
@@ -422,7 +406,7 @@ void fb_poly_set_dense(const fb_t f);
 
 /**
  * Configures a trinomial as the irreducible polynomial by its non-zero
- * coefficients. The other coefficients are FB_BITS and 0.
+ * coefficients. The other coefficients are RLC_FB_BITS and 0.
  *
  * @param[in] a				- the second coefficient.
  */
@@ -430,7 +414,7 @@ void fb_poly_set_trino(int a);
 
 /**
  * Configures a pentanomial as the binary field modulo by its non-zero
- * coefficients. The other coefficients are FB_BITS and 0.
+ * coefficients. The other coefficients are RLC_FB_BITS and 0.
  *
  * @param[in] a				- the second coefficient.
  * @param[in] b				- the third coefficient.
@@ -461,9 +445,9 @@ const dig_t *fb_poly_tab_srz(int i);
 const fb_t *fb_poly_tab_sqr(int i);
 
 /**
- * Returns an addition chain for (FB_BITS - 1).
+ * Returns an addition chain for (RLC_FB_BITS - 1).
  *
- * @param[out] len		- the number of elements in the addition chain.
+ * @param[out] len			- the number of elements in the addition chain.
  *
  * @return a pointer to the addition chain.
  */
@@ -472,7 +456,7 @@ const int *fb_poly_get_chain(int *len);
 /**
  * Returns the non-zero coefficients of the configured trinomial or pentanomial.
  * If b is -1, the irreducible polynomial configured is a trinomial.
- * The other coefficients are FB_BITS and 0.
+ * The other coefficients are RLC_FB_BITS and 0.
  *
  * @param[out] a			- the second coefficient.
  * @param[out] b			- the third coefficient.
@@ -523,15 +507,6 @@ void fb_param_print(void);
  */
 void fb_poly_add(fb_t c, const fb_t a);
 
-/**
- * Subtracts the irreducible polynomial from a binary field element. Computes
- * c = a - f(z).
- *
- * @param[out] c			- the destination.
- * @param[in] a				- the binary field element.
- */
-void fb_poly_sub(fb_t c, const fb_t a);
-
 /**
  * Copies the second argument to the first argument.
  *
@@ -584,7 +559,7 @@ void fb_set_bit(fb_t a, int bit, int value);
 /**
  * Assigns a small positive polynomial to a binary field element.
  *
- * The degree of the polynomial must be smaller than FB_DIGIT.
+ * The degree of the polynomial must be smaller than RLC_DIG.
  *
  * @param[out] c			- the result.
  * @param[in] a				- the small polynomial to assign.
@@ -656,7 +631,7 @@ void fb_write_str(char *str, int len, const fb_t a, int radix);
  * @param[out] a			- the result.
  * @param[in] bin			- the byte vector.
  * @param[in] len			- the buffer capacity.
- * @throw ERR_NO_BUFFER		- if the buffer capacity is not FP_BYTES.
+ * @throw ERR_NO_BUFFER		- if the buffer capacity is not RLC_FP_BYTES.
  */
 void fb_read_bin(fb_t a, const uint8_t *bin, int len);
 
@@ -666,7 +641,7 @@ void fb_read_bin(fb_t a, const uint8_t *bin, int len);
  * @param[out] bin			- the byte vector.
  * @param[in] len			- the buffer capacity.
  * @param[in] a				- the binary field element to write.
- * @throw ERR_NO_BUFFER		- if the buffer capacity is not FP_BYTES.
+ * @throw ERR_NO_BUFFER		- if the buffer capacity is not RLC_FP_BYTES.
  */
 void fb_write_bin(uint8_t *bin, int len, const fb_t a);
 
@@ -675,7 +650,7 @@ void fb_write_bin(uint8_t *bin, int len, const fb_t a);
  *
  * @param[in] a				- the first binary field element.
  * @param[in] b				- the second binary field element.
- * @return CMP_LT if a < b, CMP_EQ if a == b and CMP_GT if a > b.
+ * @return RLC_EQ if a == b, and RLC_NE otherwise.
  */
 int fb_cmp(const fb_t a, const fb_t b);
 
@@ -685,7 +660,7 @@ int fb_cmp(const fb_t a, const fb_t b);
  *
  * @param[in] a				- the binary field element.
  * @param[in] b				- the small binary field element.
- * @return CMP_LT if a < b, CMP_EQ if a == b and CMP_GT if a > b.
+ * @return RLC_EQ if a == b, and RLC_NE otherwise.
  */
 int fb_cmp_dig(const fb_t a, dig_t b);
 
@@ -708,25 +683,6 @@ void fb_add(fb_t c, const fb_t a, const fb_t b);
  */
 void fb_add_dig(fb_t c, const fb_t a, dig_t b);
 
-/**
- * Subtracts a binary field element from another. Computes c = a - b.
- *
- * @param[out] c			- the result.
- * @param[in] a				- the binary field element.
- * @param[in] b				- the binary field element to subtract.
- */
-void fb_sub(fb_t c, const fb_t a, const fb_t b);
-
-/**
- * Subtracts a small binary field element from a binary field element.
- * Computes c = a - b.
- *
- * @param[out] c			- the result.
- * @param[in] a				- the binary field element.
- * @param[in] b				- the small binary field element to subtract.
- */
-void fb_sub_dig(fb_t c, const fb_t a, dig_t b);
-
 /**
  * Multiples two binary field elements using Shift-and-add multiplication.
  *
@@ -746,24 +702,6 @@ void fb_mul_basic(fb_t c, const fb_t a, const fb_t b);
  */
 void fb_mul_integ(fb_t c, const fb_t a, const fb_t b);
 
-/**
- * Multiples two binary field elements using Left-to-right comb multiplication.
- *
- * @param[out] c			- the result.
- * @param[in] a				- the first binary field element to multiply.
- * @param[in] b				- the second binary field element to multiply.
- */
-void fb_mul_lcomb(fb_t c, const fb_t a, const fb_t b);
-
-/**
- * Multiples two binary field elements using Right-to-left comb multiplication.
- *
- * @param[out] c			- the result.
- * @param[in] a				- the first binary field element to multiply.
- * @param[in] b				- the second binary field element to multiply.
- */
-void fb_mul_rcomb(fb_t c, const fb_t a, const fb_t b);
-
 /**
  * Multiples two binary field elements using Lopez-Dahab multiplication.
  *
@@ -813,7 +751,7 @@ void fb_sqr_integ(fb_t c, const fb_t a);
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to square.
  */
-void fb_sqr_lutbl(fb_t c, const fb_t a);
+void fb_sqr_quick(fb_t c, const fb_t a);
 
 /**
  * Shifts a binary field element to the left. Computes c = a * z^bits mod f(z).
@@ -891,6 +829,7 @@ dig_t fb_trc_quick(const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_basic(fb_t c, const fb_t a);
 
@@ -899,6 +838,7 @@ void fb_inv_basic(fb_t c, const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_binar(fb_t c, const fb_t a);
 
@@ -907,6 +847,7 @@ void fb_inv_binar(fb_t c, const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_exgcd(fb_t c, const fb_t a);
 
@@ -915,6 +856,7 @@ void fb_inv_exgcd(fb_t c, const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_almos(fb_t c, const fb_t a);
 
@@ -923,6 +865,7 @@ void fb_inv_almos(fb_t c, const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_itoht(fb_t c, const fb_t a);
 
@@ -932,14 +875,26 @@ void fb_inv_itoht(fb_t c, const fb_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_bruch(fb_t c, const fb_t a);
 
+/**
+ * Inverts a binary field element in constant-time using
+ * the Wu-Wu-Shieh-Hwang algorithm.
+ *
+ * @param[out] c			- the result.
+ * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
+ */
+void fb_inv_ctaia(fb_t c, const fb_t a);
+
 /**
  * Inverts a binary field element using a direct call to the lower layer.
  *
  * @param[out] c			- the result.
  * @param[in] a				- the binary field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fb_inv_lower(fb_t c, const fb_t a);
 
@@ -1037,4 +992,4 @@ void fb_itr_pre_quick(fb_t *t, int b);
  */
 void fb_itr_quick(fb_t c, const fb_t a, const fb_t *t);
 
-#endif /* !RELIC_FB_H */
+#endif /* !RLC_FB_H */
