@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -32,8 +33,8 @@
  * @ingroup fp
  */
 
-#ifndef RELIC_FP_H
-#define RELIC_FP_H
+#ifndef RLC_FP_H
+#define RLC_FP_H
 
 #include "relic_dv.h"
 #include "relic_bn.h"
@@ -47,27 +48,17 @@
 /**
  * Precision in bits of a prime field element.
  */
-#define FP_BITS 	((int)FP_PRIME)
-
-/**
- * Size in bits of a digit.
- */
-#define FP_DIGIT	((int)DIGIT)
-
-/**
- * Logarithm of the digit size in base 2.
- */
-#define FP_DIG_LOG	((int)DIGIT_LOG)
+#define RLC_FP_BITS 	((int)FP_PRIME)
 
 /**
  * Size in digits of a block sufficient to store a prime field element.
  */
-#define FP_DIGS		((int)((FP_BITS)/(FP_DIGIT) + (FP_BITS % FP_DIGIT > 0)))
+#define RLC_FP_DIGS 	((int)RLC_CEIL(RLC_FP_BITS, RLC_DIG))
 
 /**
  * Size in bytes of a block sufficient to store a binary field element.
  */
-#define FP_BYTES 	((int)((FP_BITS)/8 + (FP_BITS % 8 > 0)))
+#define RLC_FP_BYTES 	((int)RLC_CEIL(RLC_FP_BITS, 8))
 
 /*
  * Finite field identifiers.
@@ -109,27 +100,39 @@ enum {
 	PRIME_511187,
 	/** NIST 521-bit fast reduction polynomial. */
 	NIST_521,
-	/** 158-bit prime for use with BN curves. */
+	/** 158-bit prime for BN curve. */
 	BN_158,
-	/** 254-bit prime provided in Nogami et al. for use with BN curves. */
+	/** 254-bit prime provided in Nogami et al. for BN curves. */
 	BN_254,
-	/** 256-bit prime provided in Barreto et al. for use with BN curves. */
+	/** 256-bit prime provided in Barreto et al. for BN curves. */
 	BN_256,
-	/** 381-bit prime with BLS curves of embedding degree 12. */
+	/** 381-bit prime for BLS curve of embedding degree 12 (Zcash). */
 	B12_381,
-	/** 382-bit prime provided by Barreto for use with BN curves. */
+	/** 382-bit prime provided by Barreto for BN curve. */
 	BN_382,
-	/** 455-bit prime for use with BLS curves of embedding degree 12. */
+	/** 446-bit prime provided by Barreto for BN curve. */
+	BN_446,
+	/** 446-bit prime for BLS curve of embedding degree 12. */
+	B12_446,
+	/** 455-bit prime for BLS curve of embedding degree 12. */
 	B12_455,
-	/** 477-bit prime for use with BLS curves of embedding degree 24. */
+	/** 477-bit prime for BLS curve of embedding degree 24. */
 	B24_477,
-	/** 508-bit prime for use with KSS curves. */
+	/** 508-bit prime for KSS16 curve. */
 	KSS_508,
-	/** 638-bit prime provided in Barreto et al. for use with BN curves. */
+	/** 511-bit prime for Optimal TNFS-secure curve. */
+	OT_511,
+	/** Random 544-bit prime for Cocks-Pinch curve with embedding degree 8. */
+	CP8_544,
+	/** 569-bit prime for KSS curve with embedding degree 54. */
+	K54_569,
+	/** 575-bit prime for BLS curve with embedding degree 48. */
+	B48_575,
+	/** 638-bit prime provided in Barreto et al. for BN curve. */
 	BN_638,
-	/** 638-bit prime generated by the Barreto-Lynn-Scott method. */
+	/** 638-bit prime for BLS curve with embedding degree 12. */
 	B12_638,
-	/** 1536-bit prime for use with supersingular curves. */
+	/** 1536-bit prime for supersingular curve with embedding degree k = 2. */
 	SS_1536,
 };
 
@@ -138,9 +141,13 @@ enum {
  * prime field elements. This can be used to avoid carries.
  */
 #if ((FP_PRIME % WSIZE) != 0) && ((FP_PRIME % WSIZE) <= (WSIZE - 2))
-#define FP_SPACE
+#if ((2 * FP_PRIME % WSIZE) != 0) && ((2 * FP_PRIME % WSIZE) <= (WSIZE - 2))
+#define RLC_FP_ROOM
 #else
-#undef FP_SPACE
+#undef RLC_FP_ROOM
+#endif
+#else
+#undef RLC_FP_ROOM
 #endif
 
 /*============================================================================*/
@@ -155,7 +162,7 @@ enum {
  * stored in the first positions of the vector.
  */
 #if ALLOC == AUTO
-typedef relic_align dig_t fp_t[FP_DIGS + PADDING(FP_BYTES)/(FP_DIGIT / 8)];
+typedef rlc_align dig_t fp_t[RLC_FP_DIGS + RLC_PAD(RLC_FP_BYTES)/(RLC_DIG / 8)];
 #else
 typedef dig_t *fp_t;
 #endif
@@ -163,7 +170,7 @@ typedef dig_t *fp_t;
 /**
  * Represents a prime field element with automatic memory allocation.
  */
-typedef relic_align dig_t fp_st[FP_DIGS + PADDING(FP_BYTES)/(FP_DIGIT / 8)];
+typedef rlc_align dig_t fp_st[RLC_FP_DIGS + RLC_PAD(RLC_FP_BYTES)/(RLC_DIG / 8)];
 
 /*============================================================================*/
 /* Macro definitions                                                          */
@@ -186,15 +193,13 @@ typedef relic_align dig_t fp_st[FP_DIGS + PADDING(FP_BYTES)/(FP_DIGIT / 8)];
  * @param[out] A			- the new prime field element.
  */
 #if ALLOC == DYNAMIC
-#define fp_new(A)			dv_new_dynam((dv_t *)&(A), FP_DIGS)
-#elif ALLOC == STATIC
-#define fp_new(A)			dv_new_statc((dv_t *)&(A), FP_DIGS)
+#define fp_new(A)			dv_new_dynam((dv_t *)&(A), RLC_FP_DIGS)
 #elif ALLOC == AUTO
 #define fp_new(A)			/* empty */
 #elif ALLOC == STACK
 #define fp_new(A)															\
-	A = (dig_t *)alloca(FP_BYTES + PADDING(FP_BYTES));						\
-	A = (dig_t *)ALIGNED(A);												\
+	A = (dig_t *)alloca(RLC_FP_BYTES + RLC_PAD(RLC_FP_BYTES));				\
+	A = (dig_t *)RLC_ALIGN(A);												\
 
 #endif
 
@@ -205,8 +210,6 @@ typedef relic_align dig_t fp_st[FP_DIGS + PADDING(FP_BYTES)/(FP_DIGIT / 8)];
  */
 #if ALLOC == DYNAMIC
 #define fp_free(A)			dv_free_dynam((dv_t *)&(A))
-#elif ALLOC == STATIC
-#define fp_free(A)			dv_free_statc((dv_t *)&(A))
 #elif ALLOC == AUTO
 #define fp_free(A)			/* empty */
 #elif ALLOC == STACK
@@ -352,6 +355,8 @@ typedef relic_align dig_t fp_st[FP_DIGS + PADDING(FP_BYTES)/(FP_DIGIT / 8)];
 #define fp_inv(C, A)	fp_inv_monty(C, A)
 #elif FP_INV == EXGCD
 #define fp_inv(C, A)	fp_inv_exgcd(C, A)
+#elif FP_INV == DIVST
+#define fp_inv(C, A)	fp_inv_divst(C, A)
 #elif FP_INV == LOWER
 #define fp_inv(C, A)	fp_inv_lower(C, A)
 #endif
@@ -416,7 +421,7 @@ dig_t fp_prime_get_mod8(void);
 
 /**
  * Returns the prime stored in special form. The most significant bit is
- * FP_BITS.
+ * RLC_FP_BITS.
  *
  * @param[out] len		- the number of returned bits, can be NULL.
  *
@@ -438,6 +443,13 @@ int fp_prime_get_qnr(void);
  */
 int fp_prime_get_cnr(void);
 
+/**
+ * Returns the 2-adicity of the prime modulus.
+ *
+ * @return the 2-adicity of the modulus.
+ */
+int fp_prime_get_2ad(void);
+
 /**
  * Returns the prime field parameter identifier.
  *
@@ -460,6 +472,12 @@ void fp_prime_set_dense(const bn_t p);
  */
 void fp_prime_set_pmers(const int *spars, int len);
 
+/**
+* Assigns the prime field modulus to a parametrization from a family of
+ * pairing-friendly curves.
+ */
+void fp_prime_set_pairf(const bn_t x, int pairf);
+
 /**
  * Computes the constants needed for evaluating Frobenius maps in higher
  * extension fields.
@@ -501,28 +519,28 @@ void fp_param_set(int param);
 /**
  * Assigns any pre-defined parameter as the prime modulus.
  *
- * @return STS_OK if no errors occurred; STS_ERR otherwise.
+ * @return RLC_OK if no errors occurred; RLC_ERR otherwise.
  */
 int fp_param_set_any(void);
 
 /**
  * Assigns the order of the prime field to any non-sparse prime.
  *
- * @return STS_OK if no errors occurred; STS_ERR otherwise.
+ * @return RLC_OK if no errors occurred; RLC_ERR otherwise.
  */
 int fp_param_set_any_dense(void);
 
 /**
  * Assigns the order of the prime field to any sparse prime.
  *
- * @return STS_OK if no errors occurred; STS_ERR otherwise.
+ * @return RLC_OK if no errors occurred; RLC_ERR otherwise.
  */
 int fp_param_set_any_pmers(void);
 
 /**
  * Assigns the order of the prime field to any towering-friendly prime.
  *
- * @return STS_OK if no errors occurred; STS_ERR otherwise.
+ * @return RLC_OK if no errors occurred; RLC_ERR otherwise.
  */
 int fp_param_set_any_tower(void);
 
@@ -532,16 +550,24 @@ int fp_param_set_any_tower(void);
 void fp_param_print(void);
 
 /**
- * Returns the variable used to parameterize the given prime modulus.
+ * Returns the variable used to parametrize the given prime modulus.
  *
  * @param[out] x			- the integer parameter.
  */
-void fp_param_get_var(bn_t x);
+void fp_prime_get_par(bn_t x);
+
+/**
+ * Returns the absolute value of the variable used to parameterize the given
+ * prime modulus in sparse form.
+ *
+ * @param[out] len			- the length of the representation.
+ */
+const int *fp_prime_get_par_sps(int *len);
 
 /**
  * Returns the absolute value of the variable used to parameterize the currently
  * configured prime modulus in sparse form. The first argument must be an array
- * of size (MAX_TERMS + 1).
+ * of size (RLC_TERMS + 1).
  *
  * @param[out] s			- the parameter in sparse form.
  * @param[out] len			- the length of the parameter in sparse form.
@@ -551,18 +577,6 @@ void fp_param_get_var(bn_t x);
  */
 void fp_param_get_sps(int *s, int *len);
 
-/**
- * Returns the value of the parameter used to construct the optimal ate pairing
- * on the currently configured elliptic curve. The first argument must be an
- * array of size FP_BITS.
- *
- * @param[out] s			- the parameter in sparse form.
- * @param[out] len			- the length of the parameter in sparse form.
- * @throw ERR_NO_BUFFER		- if the buffer capacity is insufficient.
- * @throw ERR_NO_VALID		- if the current configuration is invalid.
- */
-void fp_param_get_map(int *s, int *len);
-
 /**
  * Copies the second argument to the first argument.
  *
@@ -616,7 +630,7 @@ void fp_set_bit(fp_t a, int bit, int value);
  * Assigns a small positive constant to a prime field element.
  *
  * The constant must fit on a multiple precision digit, or dig_t type using
- * only the number of bits specified on FP_DIGIT.
+ * only the number of bits specified on RLC_DIG.
  *
  * @param[out] c			- the result.
  * @param[in] a				- the constant to assign.
@@ -687,7 +701,7 @@ void fp_write_str(char *str, int len, const fp_t a, int radix);
  * @param[out] a			- the result.
  * @param[in] bin			- the byte vector.
  * @param[in] len			- the buffer capacity.
- * @throw ERR_NO_BUFFER		- if the buffer capacity is not FP_BYTES.
+ * @throw ERR_NO_BUFFER		- if the buffer capacity is not RLC_FP_BYTES.
  */
 void fp_read_bin(fp_t a, const uint8_t *bin, int len);
 
@@ -697,7 +711,7 @@ void fp_read_bin(fp_t a, const uint8_t *bin, int len);
  * @param[out] bin			- the byte vector.
  * @param[in] len			- the buffer capacity.
  * @param[in] a				- the prime field element to write.
- * @throw ERR_NO_BUFFER		- if the buffer capacity is not FP_BYTES.
+ * @throw ERR_NO_BUFFER		- if the buffer capacity is not RLC_FP_BYTES.
  */
 void fp_write_bin(uint8_t *bin, int len, const fp_t a);
 
@@ -706,7 +720,7 @@ void fp_write_bin(uint8_t *bin, int len, const fp_t a);
  *
  * @param[in] a				- the first prime field element.
  * @param[in] b				- the second prime field element.
- * @return CMP_LT if a < b, CMP_EQ if a == b and CMP_GT if a > b.
+ * @return RLC_EQ if a == b, and RLC_NE otherwise.
  */
 int fp_cmp(const fp_t a, const fp_t b);
 
@@ -716,7 +730,7 @@ int fp_cmp(const fp_t a, const fp_t b);
  *
  * @param[in] a				- the prime field element.
  * @param[in] b				- the digit.
- * @return CMP_LT if a < b, CMP_EQ if a == b and CMP_GT if a > b.
+ * @return RLC_EQ if a == b, and RLC_NE otherwise.
  */
 int fp_cmp_dig(const fp_t a, dig_t b);
 
@@ -964,6 +978,7 @@ void fp_rdc_quick(fp_t c, dv_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fp_inv_basic(fp_t c, const fp_t a);
 
@@ -972,6 +987,7 @@ void fp_inv_basic(fp_t c, const fp_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fp_inv_binar(fp_t c, const fp_t a);
 
@@ -980,6 +996,7 @@ void fp_inv_binar(fp_t c, const fp_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fp_inv_monty(fp_t c, const fp_t a);
 
@@ -988,6 +1005,7 @@ void fp_inv_monty(fp_t c, const fp_t a);
  *
  * @param[out] c			- the result.
  * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fp_inv_exgcd(fp_t c, const fp_t a);
 
@@ -1000,11 +1018,22 @@ void fp_inv_exgcd(fp_t c, const fp_t a);
  */
 void fp_inv_exgcd_bn(bn_t c, const bn_t u, const bn_t p);
 
+/**
+ * Inverts a prime field element using the constant-time division step approach
+ * by Bernstein and Bo-Yin Yang.
+ *
+ * @param[out] c			- the result.
+ * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
+ */
+void fp_inv_divst(fp_t c, const fp_t a);
+
 /**
  * Inverts a prime field element using a direct call to the lower layer.
  *
  * @param[out] c			- the result.
  * @param[in] a				- the prime field element to invert.
+ * @throw ERR_NO_VALID		- if the field element is not invertible.
  */
 void fp_inv_lower(fp_t c, const fp_t a);
 
@@ -1056,4 +1085,4 @@ void fp_exp_monty(fp_t c, const fp_t a, const bn_t b);
  */
 int fp_srt(fp_t c, const fp_t a);
 
-#endif /* !RELIC_FP_H */
+#endif /* !RLC_FP_H */
