@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -206,7 +207,7 @@ void bn_gcd_lehme(bn_t c, const bn_t a, const bn_t b) {
 		bn_new(t2);
 		bn_new(t3);
 
-		if (bn_cmp(a, b) == CMP_GT) {
+		if (bn_cmp(a, b) == RLC_GT) {
 			bn_abs(x, a);
 			bn_abs(y, b);
 		} else {
@@ -214,9 +215,9 @@ void bn_gcd_lehme(bn_t c, const bn_t a, const bn_t b) {
 			bn_abs(y, a);
 		}
 		while (y->used > 1) {
-			bn_rsh(u, x, bn_bits(x) - BN_DIGIT);
+			bn_rsh(u, x, bn_bits(x) - RLC_DIG);
 			_x = u->dp[0];
-			bn_rsh(v, y, bn_bits(x) - BN_DIGIT);
+			bn_rsh(v, y, bn_bits(x) - RLC_DIG);
 			_y = v->dp[0];
 			_a = _d = 1;
 			_b = _c = 0;
@@ -225,11 +226,11 @@ void bn_gcd_lehme(bn_t c, const bn_t a, const bn_t b) {
 				q = _x / _y;
 				t = _x % _y;
 			}
-			if (t >= ((dig_t)1 << (BN_DIGIT / 2))) {
+			if (t >= ((dig_t)1 << (RLC_DIG / 2))) {
 				while (1) {
 					_q = _y / t;
 					_t = _y % t;
-					if (_t < ((dig_t)1 << (BN_DIGIT / 2))) {
+					if (_t < ((dig_t)1 << (RLC_DIG / 2))) {
 						break;
 					}
 					_x = _y;
@@ -249,8 +250,8 @@ void bn_gcd_lehme(bn_t c, const bn_t a, const bn_t b) {
 				bn_copy(x, y);
 				bn_copy(y, t0);
 			} else {
-				bn_rsh(u, x, bn_bits(x) - 2 * BN_DIGIT);
-				bn_rsh(v, y, bn_bits(x) - 2 * BN_DIGIT);
+				bn_rsh(u, x, bn_bits(x) - 2 * RLC_DIG);
+				bn_rsh(v, y, bn_bits(x) - 2 * RLC_DIG);
 				if (_a < 0) {
 					bn_mul_dig(t0, u, -_a);
 					bn_neg(t0, t0);
@@ -277,20 +278,20 @@ void bn_gcd_lehme(bn_t c, const bn_t a, const bn_t b) {
 				}
 				bn_add(u, t0, t1);
 				bn_add(v, t2, t3);
-				bn_rsh(t0, u, bn_bits(u) - BN_DIGIT);
+				bn_rsh(t0, u, bn_bits(u) - RLC_DIG);
 				_x = t0->dp[0];
-				bn_rsh(t1, v, bn_bits(u) - BN_DIGIT);
+				bn_rsh(t1, v, bn_bits(u) - RLC_DIG);
 				_y = t1->dp[0];
 				t = 0;
 				if (_y != 0) {
 					q = _x / _y;
 					t = _x % _y;
 				}
-				if (t >= ((dig_t)1 << BN_DIGIT / 2)) {
+				if (t >= ((dig_t)1 << RLC_DIG / 2)) {
 					while (1) {
 						_q = _y / t;
 						_t = _y % t;
-						if (_t < ((dig_t)1 << BN_DIGIT / 2)) {
+						if (_t < ((dig_t)1 << RLC_DIG / 2)) {
 							break;
 						}
 						_x = _y;
@@ -398,7 +399,7 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 		bn_new(t3);
 		bn_new(t4);
 
-		if (bn_cmp(a, b) != CMP_LT) {
+		if (bn_cmp(a, b) != RLC_LT) {
 			bn_abs(x, a);
 			bn_abs(y, b);
 			swap = 0;
@@ -412,9 +413,9 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 		bn_set_dig(d, 1);
 
 		while (y->used > 1) {
-			bn_rsh(u, x, bn_bits(x) - BN_DIGIT);
+			bn_rsh(u, x, bn_bits(x) - RLC_DIG);
 			_x = u->dp[0];
-			bn_rsh(v, y, bn_bits(x) - BN_DIGIT);
+			bn_rsh(v, y, bn_bits(x) - RLC_DIG);
 			_y = v->dp[0];
 			_a = _d = 1;
 			_b = _c = 0;
@@ -423,11 +424,11 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 				q = _x / _y;
 				t = _x % _y;
 			}
-			if (t >= ((dig_t)1 << (BN_DIGIT / 2))) {
+			if (t >= ((dig_t)1 << (RLC_DIG / 2))) {
 				while (1) {
 					_q = _y / t;
 					_t = _y % t;
-					if (_t < ((dig_t)1 << (BN_DIGIT / 2))) {
+					if (_t < ((dig_t)1 << (RLC_DIG / 2))) {
 						break;
 					}
 					_x = _y;
@@ -451,8 +452,8 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 				bn_copy(t4, d);
 				bn_copy(d, t1);
 			} else {
-				bn_rsh(u, x, bn_bits(x) - 2 * BN_DIGIT);
-				bn_rsh(v, y, bn_bits(x) - 2 * BN_DIGIT);
+				bn_rsh(u, x, bn_bits(x) - 2 * RLC_DIG);
+				bn_rsh(v, y, bn_bits(x) - 2 * RLC_DIG);
 				if (_a < 0) {
 					bn_mul_dig(t0, u, -_a);
 					bn_neg(t0, t0);
@@ -479,20 +480,20 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 				}
 				bn_add(u, t0, t1);
 				bn_add(v, t2, t3);
-				bn_rsh(t0, u, bn_bits(u) - BN_DIGIT);
+				bn_rsh(t0, u, bn_bits(u) - RLC_DIG);
 				_x = t0->dp[0];
-				bn_rsh(t1, v, bn_bits(u) - BN_DIGIT);
+				bn_rsh(t1, v, bn_bits(u) - RLC_DIG);
 				_y = t1->dp[0];
 				t = 0;
 				if (_y != 0) {
 					q = _x / _y;
 					t = _x % _y;
 				}
-				if (t >= ((dig_t)1 << BN_DIGIT / 2)) {
+				if (t >= ((dig_t)1 << RLC_DIG / 2)) {
 					while (1) {
 						_q = _y / t;
 						_t = _y % t;
-						if (_t < ((dig_t)1 << BN_DIGIT / 2)) {
+						if (_t < ((dig_t)1 << RLC_DIG / 2)) {
 							break;
 						}
 						_x = _y;
@@ -570,9 +571,6 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 			bn_mul(x, b, t4);
 			bn_sub(x, c, x);
 			bn_div(d, x, a);
-			if (bn_sign(x) != bn_sign(a)) {
-				bn_add_dig(d, d, 1);
-			}
 		} else {
 			bn_mul(t0, t4, u);
 			bn_mul(t1, d, v);
@@ -580,9 +578,6 @@ void bn_gcd_ext_lehme(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 			bn_mul(x, a, d);
 			bn_sub(x, c, x);
 			bn_div(t4, x, b);
-			if (bn_sign(x) != bn_sign(b)) {
-				bn_add_dig(t4, t4, 1);
-			}
 		}
 		if (e != NULL) {
 			bn_copy(e, t4);
@@ -650,7 +645,7 @@ void bn_gcd_stein(bn_t c, const bn_t a, const bn_t b) {
 			bn_sub(t, u, v);
 			bn_abs(t, t);
 			bn_hlv(t, t);
-			if (bn_cmp(u, v) != CMP_LT) {
+			if (bn_cmp(u, v) != RLC_LT) {
 				bn_copy(u, t);
 			} else {
 				bn_copy(v, t);
@@ -761,7 +756,7 @@ void bn_gcd_ext_stein(bn_t c, bn_t d, bn_t e, const bn_t a, const bn_t b) {
 				}
 			}
 			/* If u >= v then u = u - v, A = A - C, B = B - D. */
-			if (bn_cmp(u, v) != CMP_LT) {
+			if (bn_cmp(u, v) != RLC_LT) {
 				bn_sub(u, u, v);
 				bn_sub(_a, _a, d);
 				bn_sub(_b, _b, _e);
@@ -838,7 +833,7 @@ void bn_gcd_ext_mid(bn_t c, bn_t d, bn_t e, bn_t f, const bn_t a, const bn_t b)
 		bn_new(y);
 		bn_new(z);
 
-		if (bn_cmp_abs(a, b) == CMP_GT) {
+		if (bn_cmp_abs(a, b) == RLC_GT) {
 			bn_abs(u, a);
 			bn_abs(v, b);
 		} else {
@@ -868,7 +863,7 @@ void bn_gcd_ext_mid(bn_t c, bn_t d, bn_t e, bn_t f, const bn_t a, const bn_t b)
 				bn_neg(f, x);
 				wait = 0;
 			}
-			if (bn_cmp(u, p) == CMP_GT) {
+			if (bn_cmp(u, p) == RLC_GT) {
 				bn_copy(c, r);
 				bn_neg(d, x);
 				bn_copy(w, u);
@@ -887,7 +882,7 @@ void bn_gcd_ext_mid(bn_t c, bn_t d, bn_t e, bn_t f, const bn_t a, const bn_t b)
 		bn_add(q, q, r);
 
 		/* Output (e, f) as the vector of smaller norm. */
-		if (bn_cmp(t, q) == CMP_LT) {
+		if (bn_cmp(t, q) == RLC_LT) {
 			bn_copy(e, w);
 			bn_copy(f, y);
 		}
