@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -46,28 +47,28 @@ static char get_bits(const bn_t a, int from, int to) {
 	int f, t;
 	dig_t mf, mt;
 
-	SPLIT(from, f, from, BN_DIG_LOG);
-	SPLIT(to, t, to, BN_DIG_LOG);
+	RLC_RIP(from, f, from);
+	RLC_RIP(to, t, to);
 
 	if (f == t) {
 		/* Same digit. */
 
-		mf = MASK(from);
-		mt = MASK(to + 1);
-
-		if (to + 1 == BN_DIGIT) {
-			mt = DMASK;
+		mf = RLC_MASK(from);
+		if (to + 1 >= RLC_DIG) {
+			mt = RLC_DMASK;
+		} else {
+			mt = RLC_MASK(to + 1);
 		}
 
 		mf = mf ^ mt;
 
 		return ((a->dp[f] & (mf)) >> from);
 	} else {
-		mf = MASK(BN_DIGIT - from) << from;
-		mt = MASK(to + 1);
+		mf = RLC_MASK(RLC_DIG - from) << from;
+		mt = RLC_MASK(to + 1);
 
 		return ((a->dp[f] & mf) >> from) |
-				((a->dp[t] & mt) << (BN_DIGIT - from));
+				((a->dp[t] & mt) << (RLC_DIG - from));
 	}
 }
 
@@ -95,7 +96,7 @@ void bn_rec_win(uint8_t *win, int *len, const bn_t k, int w) {
 
 	l = bn_bits(k);
 
-	if (*len < CEIL(l, w)) {
+	if (*len < RLC_CEIL(l, w)) {
 		THROW(ERR_NO_BUFFER);
 	}
 
@@ -112,7 +113,7 @@ void bn_rec_slw(uint8_t *win, int *len, const bn_t k, int w) {
 
 	l = bn_bits(k);
 
-	if (*len < CEIL(l, w)) {
+	if (*len < RLC_CEIL(l, w)) {
 		THROW(ERR_NO_BUFFER);
 	}
 
@@ -123,7 +124,7 @@ void bn_rec_slw(uint8_t *win, int *len, const bn_t k, int w) {
 			i--;
 			win[j++] = 0;
 		} else {
-			s = MAX(i - w + 1, 0);
+			s = RLC_MAX(i - w + 1, 0);
 			while (!bn_get_bit(k, s)) {
 				s++;
 			}
@@ -150,7 +151,7 @@ void bn_rec_naf(int8_t *naf, int *len, const bn_t k, int w) {
 		bn_new(t);
 		bn_abs(t, k);
 
-		mask = MASK(w);
+		mask = RLC_MASK(w);
 		l = (1 << w);
 
 		i = 0;
@@ -411,7 +412,7 @@ void bn_rec_tnaf_mod(bn_t r0, bn_t r1, const bn_t k, int u, int m) {
 void bn_rec_tnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 	int i, l;
 	bn_t tmp, r0, r1;
-	int8_t beta[1 << (w - 2)], gama[1 << (w - 2)];
+	int8_t beta[64], gama[64];
 	uint8_t t_w;
 	dig_t t0, t1, mask;
 	int s, t, u_i;
@@ -433,7 +434,7 @@ void bn_rec_tnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 		bn_abs(tmp, k);
 		bn_rec_tnaf_mod(r0, r1, tmp, u, m);
 
-		mask = MASK(w);
+		mask = RLC_MASK(w);
 		l = 1 << w;
 
 		i = 0;
@@ -455,11 +456,11 @@ void bn_rec_tnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 			/* If r0 is odd. */
 			if (w == 2) {
 				t0 = r0->dp[0];
-				if (bn_sign(r0) == BN_NEG) {
+				if (bn_sign(r0) == RLC_NEG) {
 					t0 = l - t0;
 				}
 				t1 = r1->dp[0];
-				if (bn_sign(r1) == BN_NEG) {
+				if (bn_sign(r1) == RLC_NEG) {
 					t1 = l - t1;
 				}
 				u_i = 2 - ((t0 - 2 * t1) & mask);
@@ -472,12 +473,12 @@ void bn_rec_tnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 			} else {
 				/* t0 = r0 mod_s 2^w. */
 				t0 = r0->dp[0];
-				if (bn_sign(r0) == BN_NEG) {
+				if (bn_sign(r0) == RLC_NEG) {
 					t0 = l - t0;
 				}
 				/* t1 = r1 mod_s 2^w. */
 				t1 = r1->dp[0];
-				if (bn_sign(r1) == BN_NEG) {
+				if (bn_sign(r1) == RLC_NEG) {
 					t1 = l - t1;
 				}
 				/* u = r0 + r1 * (t_w) mod_s 2^w. */
@@ -537,7 +538,7 @@ void bn_rec_tnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 void bn_rec_rtnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w) {
 	int i, l;
 	bn_t tmp, r0, r1;
-	int8_t beta[1 << (w - 2)], gama[1 << (w - 2)];
+	int8_t beta[64], gama[64];
 	uint8_t t_w;
 	dig_t t0, t1, mask;
 	int s, t, u_i;
@@ -558,19 +559,19 @@ void bn_rec_rtnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w)
 		bn_rec_tnaf_get(&t_w, beta, gama, u, w);
 		bn_abs(tmp, k);
 		bn_rec_tnaf_mod(r0, r1, tmp, u, m);
-		mask = MASK(w);
-		l = CEIL(m + 2, (w - 1));
+		mask = RLC_MASK(w);
+		l = RLC_CEIL(m + 2, (w - 1));
 
 		i = 0;
 		while (i < l) {
 			/* If r0 is odd. */
 			if (w == 2) {
 				t0 = r0->dp[0];
-				if (bn_sign(r0) == BN_NEG) {
+				if (bn_sign(r0) == RLC_NEG) {
 					t0 = (1 << w) - t0;
 				}
 				t1 = r1->dp[0];
-				if (bn_sign(r1) == BN_NEG) {
+				if (bn_sign(r1) == RLC_NEG) {
 					t1 = (1 << w) - t1;
 				}
 				u_i = ((t0 - 2 * t1) & mask) - 2;
@@ -583,12 +584,12 @@ void bn_rec_rtnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w)
 			} else {
 				/* t0 = r0 mod_s 2^w. */
 				t0 = r0->dp[0];
-				if (bn_sign(r0) == BN_NEG) {
+				if (bn_sign(r0) == RLC_NEG) {
 					t0 = (1 << w) - t0;
 				}
 				/* t1 = r1 mod_s 2^w. */
 				t1 = r1->dp[0];
-				if (bn_sign(r1) == BN_NEG) {
+				if (bn_sign(r1) == RLC_NEG) {
 					t1 = (1 << w) - t1;
 				}
 				/* u = r0 + r1 * (t_w) mod_s 2^w. */
@@ -635,10 +636,10 @@ void bn_rec_rtnaf(int8_t *tnaf, int *len, const bn_t k, int8_t u, int m, int w)
 		}
 		s = r0->dp[0];
 		t = r1->dp[0];
-		if (bn_sign(r0) == BN_NEG) {
+		if (bn_sign(r0) == RLC_NEG) {
 			s = -s;
 		}
-		if (bn_sign(r1) == BN_NEG) {
+		if (bn_sign(r1) == RLC_NEG) {
 			t = -t;
 		}
 		if (s != 0 && t != 0) {
@@ -681,10 +682,10 @@ void bn_rec_reg(int8_t *naf, int *len, const bn_t k, int n, int w) {
 
 	bn_null(t);
 
-	mask = MASK(w);
-	l = CEIL(n, (w - 1));
+	mask = RLC_MASK(w);
+	l = RLC_CEIL(n, w - 1);
 
-	if (*len < l) {
+	if (*len <= l) {
 		THROW(ERR_NO_BUFFER);
 	}
 
@@ -694,23 +695,23 @@ void bn_rec_reg(int8_t *naf, int *len, const bn_t k, int n, int w) {
 
 		i = 0;
 		if (w == 2) {
-			for (i = 0; i < l; i++, naf++) {
+			for (i = 0; i < l; i++) {
 				u_i = (t->dp[0] & mask) - 2;
 				t->dp[0] -= u_i;
-				*naf = u_i;
+				naf[i] = u_i;
 				bn_hlv(t, t);
 			}
 			bn_get_dig(&t0, t);
-			*naf = t0;
+			naf[i] = t0;
 		} else {
-			for (i = 0; i < l; i++, naf++) {
+			for (i = 0; i < l; i++) {
 				u_i = (t->dp[0] & mask) - (1 << (w - 1));
 				t->dp[0] -= u_i;
-				*naf = u_i;
+				naf[i] = u_i;
 				bn_rsh(t, t, w - 1);
 			}
 			bn_get_dig(&t0, t);
-			*naf = t0;
+			naf[i] = t0;
 		}
 		*len = l + 1;
 	}
@@ -744,7 +745,7 @@ void bn_rec_jsf(int8_t *jsf, int *len, const bn_t k, const bn_t l) {
 
 		i = bn_bits(k);
 		j = bn_bits(l);
-		offset = MAX(i, j) + 1;
+		offset = RLC_MAX(i, j) + 1;
 
 		i = 0;
 		d0 = d1 = 0;
@@ -752,14 +753,14 @@ void bn_rec_jsf(int8_t *jsf, int *len, const bn_t k, const bn_t l) {
 			bn_get_dig(&l0, n0);
 			bn_get_dig(&l1, n1);
 			/* For reduction modulo 8. */
-			l0 = (l0 + d0) & MASK(3);
-			l1 = (l1 + d1) & MASK(3);
+			l0 = (l0 + d0) & RLC_MASK(3);
+			l1 = (l1 + d1) & RLC_MASK(3);
 
 			if (l0 % 2 == 0) {
 				u0 = 0;
 			} else {
-				u0 = 2 - (l0 & MASK(2));
-				if ((l0 == 3 || l0 == 5) && ((l1 & MASK(2)) == 2)) {
+				u0 = 2 - (l0 & RLC_MASK(2));
+				if ((l0 == 3 || l0 == 5) && ((l1 & RLC_MASK(2)) == 2)) {
 					u0 = (int8_t)-u0;
 				}
 			}
@@ -767,8 +768,8 @@ void bn_rec_jsf(int8_t *jsf, int *len, const bn_t k, const bn_t l) {
 			if (l1 % 2 == 0) {
 				u1 = 0;
 			} else {
-				u1 = 2 - (l1 & MASK(2));
-				if ((l1 == 3 || l1 == 5) && ((l0 & MASK(2)) == 2)) {
+				u1 = 2 - (l1 & RLC_MASK(2));
+				if ((l1 == 3 || l1 == 5) && ((l0 & RLC_MASK(2)) == 2)) {
 					u1 = (int8_t)-u1;
 				}
 			}
