@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -35,19 +36,23 @@
 /* Private definitions                                                        */
 /*============================================================================*/
 
-#if FP_PRIME == 1536
-/**
- * Cofactor description of 1536-bit prime modulus.
- */
-#define SS_P1536	"83093742908D4D529CEF06C72191A05D5E6073FE861E637D7747C3E52FBB92DAA5DDF3EF1C61F5F70B256802481A36CAFE995FE33CD54014B846751364C0D3B8327D9E45366EA08F1B3446AC23C9D4B656886731A8D05618CFA1A3B202A2445ABA0E77C5F4F00CA1239975A05377084F256DEAA07D21C4CF2A4279BC117603ACB7B10228C3AB8F8C1742D674395701BB02071A88683041D9C4231E8EE982B8DA"
-#endif
-
 #if FP_PRIME == 256
-
 /**
  * Random prime modulus for the Brainpool P256r1.
  */
-#define BSI_P256		"A9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377"
+#define STR_P256	"A9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377"
+
+#elif FP_PRIME == 544
+/**
+ * Random prime modulus for the Cocks-Pinch curve of embedding degree 8.
+ */
+#define STR_P544	"BB9DFD549299F1C803DDD5D7C05E7CC0373D9B1AC15B47AA5AA84626F33E58FE66943943049031AE4CA1D2719B3A84FA363BCD2539A5CD02C6F4B6B645A58C1085E14411"
+
+#elif FP_PRIME == 1536
+/**
+ * Cofactor description of 1536-bit prime modulus.
+ */
+#define STR_P1536	"83093742908D4D529CEF06C72191A05D5E6073FE861E637D7747C3E52FBB92DAA5DDF3EF1C61F5F70B256802481A36CAFE995FE33CD54014B846751364C0D3B8327D9E45366EA08F1B3446AC23C9D4B656886731A8D05618CFA1A3B202A2445ABA0E77C5F4F00CA1239975A05377084F256DEAA07D21C4CF2A4279BC117603ACB7B10228C3AB8F8C1742D674395701BB02071A88683041D9C4231E8EE982B8DA"
 
 #endif
 
@@ -59,298 +64,6 @@ int fp_param_get(void) {
 	return core_get()->fp_id;
 }
 
-void fp_param_get_var(bn_t x) {
-	bn_t a;
-
-	bn_null(a);
-
-	TRY {
-		bn_new(a);
-
-		switch (fp_param_get()) {
-			case BN_158:
-				/* x = 0x4000000031. */
-				bn_set_2b(x, 38);
-				bn_add_dig(x, x, 0x31);
-				break;
-			case BN_254:
-				/* x = -(2^62 + 2^55 + 1). */
-				bn_set_2b(x, 62);
-				bn_set_2b(a, 55);
-				bn_add(x, x, a);
-				bn_add_dig(x, x, 1);
-				bn_neg(x, x);
-				break;
-			case BN_256:
-				/* x = -0x600000000000219B. */
-				bn_set_2b(x, 62);
-				bn_set_2b(a, 61);
-				bn_add(x, x, a);
-				bn_set_dig(a, 0x21);
-				bn_lsh(a, a, 8);
-				bn_add(x, x, a);
-				bn_add_dig(x, x, 0x9B);
-				bn_neg(x, x);
-				break;
-			case B12_381:
-				/* x = -(2^63 + 2^62 + 2^60 + 2^57 + 2^48 + 2^16). */
-				bn_set_2b(x, 63);
-				bn_set_2b(a, 62);
-				bn_add(x, x, a);
-				bn_set_2b(a, 60);
-				bn_add(x, x, a);
-				bn_set_2b(a, 57);
-				bn_add(x, x, a);
-				bn_set_2b(a, 48);
-				bn_add(x, x, a);
-				bn_set_2b(a, 16);
-				bn_add(x, x, a);
-				bn_neg(x, x);
-				break;
-			case BN_382:
-				/* x = -(2^94 + 2^78 + 2^67 + 2^64 + 2^48 + 1). */
-				bn_set_2b(x, 94);
-				bn_set_2b(a, 78);
-				bn_add(x, x, a);
-				bn_set_2b(a, 67);
-				bn_add(x, x, a);
-				bn_set_2b(a, 64);
-				bn_add(x, x, a);
-				bn_set_2b(a, 48);
-				bn_add(x, x, a);
-				bn_add_dig(x, x, 1);
-				bn_neg(x, x);
-				break;
-			case B12_455:
-				/* x = 2^76 + 2^53 + 2^31 + 2^11. */
-				bn_set_2b(x, 76);
-				bn_set_2b(a, 53);
-				bn_add(x, x, a);
-				bn_set_2b(a, 31);
-				bn_add(x, x, a);
-				bn_set_2b(a, 11);
-				bn_add(x, x, a);
-				break;
-			case B24_477:
-				/* x = -2^48 + 2^45 + 2^31 - 2^7. */
-				bn_set_2b(x, 48);
-				bn_set_2b(a, 45);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 31);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 7);
-				bn_add(x, x, a);
-				bn_neg(x, x);
-				break;
-			case KSS_508:
-				/* x = -(2^64 + 2^51 - 2^46 - 2^12). */
-				bn_set_2b(x, 64);
-				bn_set_2b(a, 51);
-				bn_add(x, x, a);
-				bn_set_2b(a, 46);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 12);
-				bn_sub(x, x, a);
-				bn_neg(x, x);
-				break;
-			case BN_638:
-				/* x = 2^158 - 2^128 - 2^68 + 1. */
-				bn_set_2b(x, 158);
-				bn_set_2b(a, 128);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 68);
-				bn_sub(x, x, a);
-				bn_add_dig(x, x, 1);
-				break;
-			case B12_638:
-				/* x = -2^107 + 2^105 + 2^93 + 2^5. */
-				bn_set_2b(x, 107);
-				bn_set_2b(a, 105);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 93);
-				bn_sub(x, x, a);
-				bn_set_2b(a, 5);
-				bn_sub(x, x, a);
-				bn_neg(x, x);
-				break;
-			case SS_1536:
-				/* x = 2^255 + 2^41 + 1. */
-				bn_set_2b(x, 255);
-				bn_set_2b(a, 41);
-				bn_add(x, x, a);
-				bn_add_dig(x, x, 1);
-				break;
-			default:
-				THROW(ERR_NO_VALID);
-				break;
-		}
-	}
-	CATCH_ANY {
-		THROW(ERR_CAUGHT);
-	}
-	FINALLY {
-		bn_free(a);
-	}
-}
-
-void fp_param_get_sps(int *s, int *len) {
-	bn_t a;
-
-	bn_null(a);
-
-	if (*len < MAX_TERMS) {
-		THROW(ERR_NO_BUFFER);
-	}
-
-	TRY {
-		bn_new(a);
-
-		*len = 0;
-
-		switch (fp_param_get()) {
-			case BN_158:
-			case BN_254:
-			case BN_256:
-			case BN_382:
-				fp_param_get_var(a);
-				if (bn_sign(a) == BN_NEG) {
-					bn_neg(a, a);
-				}
-				*len = bn_ham(a);
-				for (int i = 0, j = 0; j < bn_bits(a); j++) {
-					if (bn_get_bit(a, j)) {
-						s[i++] = j;
-					}
-				}
-				break;
-			case B12_381:
-				s[0] = 16;
-				s[1] = 48;
-				s[2] = 57;
-				s[3] = 60;
-				s[4] = 62;
-				s[5] = 63;
-				*len = 6;
-				break;
-			case B12_455:
-				s[0] = 11;
-				s[1] = 31;
-				s[2] = 53;
-				s[3] = 76;
-				*len = 4;
-				break;
-			case B24_477:
-				s[0] = 7;
-				s[1] = -31;
-				s[2] = -45;
-				s[3] = 48;
-				*len = 4;
-				break;
-			case KSS_508:
-				s[0] = -12;
-				s[1] = -46;
-				s[2] = 51;
-				s[3] = 64;
-				*len = 4;
-				break;
-			case BN_638:
-				s[0] = 0;
-				s[1] = -68;
-				s[2] = -128;
-				s[3] = 158;
-				*len = 4;
-				break;
-			case B12_638:
-				s[0] = -5;
-				s[1] = -93;
-				s[2] = -105;
-				s[3] = 107;
-				*len = 4;
-				break;
-			case SS_1536:
-				s[0] = 0;
-				s[1] = 41;
-				s[2] = 255;
-				*len = 3;
-				break;
-			default:
-				THROW(ERR_NO_VALID);
-				break;
-		}
-	}
-	CATCH_ANY {
-		THROW(ERR_CAUGHT);
-	}
-	FINALLY {
-		bn_free(a);
-	}
-}
-
-void fp_param_get_map(int *s, int *len) {
-	if (*len < FP_BITS) {
-		THROW(ERR_NO_BUFFER);
-	}
-
-	for (int i = 0; i < FP_BITS; i++) {
-		s[i] = 0;
-	}
-
-	switch (fp_param_get()) {
-		case BN_158:
-			s[3] = s[5] = s[8] = s[39] = s[40] = 1;
-			*len = 41;
-			break;
-		case BN_254:
-			s[2] = s[56] = s[57] = s[63] = s[64] = 1;
-			*len = 65;
-			break;
-		case BN_256:
-			s[5] = s[7] = s[8] = s[11] = s[14] = s[15] = s[62] = s[65] = 1;
-			*len = 66;
-			break;
-		case BN_382:
-			s[2] = s[49] = s[50] = s[65] = s[66] = s[68] = 1;
-			s[69] = s[79] = s[80] = s[95] = s[96] = 1;
-			*len = 97;
-			break;
-		case B12_381:
-			s[16] = s[48] = s[57] = s[60] = s[62] = s[63] = 1;
-			*len = 64;
-			break;
-		case B12_455:
-			s[11] = s[31] = s[53] = s[76] = 1;
-			*len = 77;
-			break;
-		case B24_477:
-			s[7] = s[48] = 1;
-			s[31] = s[45] = -1;
-			*len = 49;
-			break;
-		case KSS_508:
-			s[64] = s[51] = 1;
-			s[12] = s[46] = -1;
-			*len = 65;
-			break;
-		case BN_638:
-			s[3] = s[159] = s[160] = 1;
-			s[69] = s[70] = s[129] = s[130] = -1;
-			*len = 161;
-			break;
-		case B12_638:
-			s[5] = s[93] = s[105] = -1;
-			s[107] = 1;
-			*len = 108;
-			break;
-		case SS_1536:
-			s[0] = s[41] = s[255] = 1;
-			*len = 256;
-			break;
-		default:
-			THROW(ERR_NO_VALID);
-			break;
-	}
-}
-
 void fp_param_set(int param) {
 	bn_t t0, t1, t2, p;
 	int f[10] = { 0 };
@@ -374,35 +87,21 @@ void fp_param_set(int param) {
 		switch (param) {
 #if FP_PRIME == 158
 			case BN_158:
-				/* x = 4000000031. */
-				fp_param_get_var(t0);
-				/* p = 36 * x^4 + 36 * x^3 + 24 * x^2 + 6 * x + 1. */
-				bn_set_dig(p, 1);
-				bn_mul_dig(t1, t0, 6);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 24);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul(t1, t1, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				bn_mul(t0, t0, t0);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				fp_prime_set_dense(p);
+				/* x = 0x4000000031. */
+				bn_set_2b(t0, 38);
+				bn_add_dig(t0, t0, 0x31);
+				fp_prime_set_pairf(t0, EP_BN);
 				break;
 #elif FP_PRIME == 160
 			case SECG_160:
-				/* p = 2^160 - 2^31 + 1. */
+				/* p = 2^160 - 2^31 - 1. */
 				f[0] = -1;
 				f[1] = -31;
 				f[2] = 160;
 				fp_prime_set_pmers(f, 3);
 				break;
 			case SECG_160D:
-				/* p = 2^160 - 2^32 - 2^14 - 2^12 - 2^9 - 2^8 - 2^7 - 2^3 - 2^2 - 1.*/
+				/* p = 2^160 - 2^32 - 2^14 - 2^12 - 2^9 - 2^8 - 2^7 - 2^3 - 2^2 - 1. */
 				f[0] = -1;
 				f[1] = -2;
 				f[2] = -3;
@@ -424,7 +123,7 @@ void fp_param_set(int param) {
 				fp_prime_set_pmers(f, 3);
 				break;
 			case SECG_192:
-				/* p = 2^192 - 2^32 - 2^12 - 2^8 - 2^7 - 2^6 - 2^3 - 1.*/
+				/* p = 2^192 - 2^32 - 2^12 - 2^8 - 2^7 - 2^6 - 2^3 - 1. */
 				f[0] = -1;
 				f[1] = -3;
 				f[2] = -6;
@@ -476,24 +175,12 @@ void fp_param_set(int param) {
 				break;
 #elif FP_PRIME == 254
 			case BN_254:
-				/* x = -4080000000000001. */
-				fp_param_get_var(t0);
-				/* p = 36 * x^4 + 36 * x^3 + 24 * x^2 + 6 * x + 1. */
-				bn_set_dig(p, 1);
-				bn_mul_dig(t1, t0, 6);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 24);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul(t1, t1, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				bn_mul(t0, t0, t0);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				fp_prime_set_dense(p);
+				/* x = -(2^62 + 2^55 + 1). */
+				bn_set_2b(t0, 62);
+				bn_set_bit(t0, 55, 1);
+				bn_add_dig(t0, t0, 1);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_BN);
 				break;
 #elif FP_PRIME == 255
 			case PRIME_25519:
@@ -512,7 +199,7 @@ void fp_param_set(int param) {
 				fp_prime_set_pmers(f, 5);
 				break;
 			case BSI_256:
-				bn_read_str(p, BSI_P256, strlen(BSI_P256), 16);
+				bn_read_str(p, STR_P256, strlen(STR_P256), 16);
 				fp_prime_set_dense(p);
 				break;
 			case SECG_256:
@@ -529,39 +216,26 @@ void fp_param_set(int param) {
 				break;
 			case BN_256:
 				/* x = -0x600000000000219B. */
-				fp_param_get_var(t0);
-				/* p = 36 * x^4 + 36 * x^3 + 24 * x^2 + 6 * x + 1. */
-				bn_set_dig(p, 1);
-				bn_mul_dig(t1, t0, 6);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 24);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul(t1, t1, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				bn_mul(t0, t0, t0);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				fp_prime_set_dense(p);
+				bn_set_2b(t0, 62);
+				bn_set_bit(t0, 61, 1);
+				bn_set_dig(t1, 0x21);
+				bn_lsh(t1, t1, 8);
+				bn_add(t0, t0, t1);
+				bn_add_dig(t0, t0, 0x9B);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_BN);
 				break;
 #elif FP_PRIME == 381
 			case B12_381:
-				fp_param_get_var(t0);
-				/* p = (x^2 - 2x + 1) * (x^4 - x^2 + 1)/3 + x. */
-				bn_sqr(t1, t0);
-				bn_sqr(p, t1);
-				bn_sub(p, p, t1);
-				bn_add_dig(p, p, 1);
-				bn_sub(t1, t1, t0);
-				bn_sub(t1, t1, t0);
-				bn_add_dig(t1, t1, 1);
-				bn_mul(p, p, t1);
-				bn_div_dig(p, p, 3);
-				bn_add(p, p, t0);
-				fp_prime_set_dense(p);
+				/* x = -(2^63 + 2^62 + 2^60 + 2^57 + 2^48 + 2^16). */
+				bn_set_2b(t0, 63);
+				bn_set_bit(t0, 62, 1);
+				bn_set_bit(t0, 60, 1);
+				bn_set_bit(t0, 57, 1);
+				bn_set_bit(t0, 48, 1);
+				bn_set_bit(t0, 16, 1);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_B12);
 				break;
 #elif FP_PRIME == 382
 			case PRIME_382105:
@@ -570,24 +244,15 @@ void fp_param_set(int param) {
 				fp_prime_set_dense(p);
 				break;
 			case BN_382:
-				/* x = -400040090001000000000001. */
-				fp_param_get_var(t0);
-				/* p = 36 * x^4 + 36 * x^3 + 24 * x^2 + 6 * x + 1. */
-				bn_set_dig(p, 1);
-				bn_mul_dig(t1, t0, 6);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 24);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul(t1, t1, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				bn_mul(t0, t0, t0);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				fp_prime_set_dense(p);
+				/* x = -(2^94 + 2^78 + 2^67 + 2^64 + 2^48 + 1). */
+				bn_set_2b(t0, 94);
+				bn_set_bit(t0, 78, 1);
+				bn_set_bit(t0, 67, 1);
+				bn_set_bit(t0, 64, 1);
+				bn_set_bit(t0, 48, 1);
+				bn_add_dig(t0, t0, 1);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_BN);
 				break;
 #elif FP_PRIME == 383
 			case PRIME_383187:
@@ -605,25 +270,47 @@ void fp_param_set(int param) {
 				f[4] = 384;
 				fp_prime_set_pmers(f, 5);
 				break;
+#elif FP_PRIME == 446
+			case BN_446:
+				/* x = 2^110 + 2^36 + 1. */
+				bn_set_2b(t0, 110);
+				bn_set_bit(t0, 36, 1);
+				bn_add_dig(t0, t0, 1);
+				fp_prime_set_pairf(t0, EP_BN);
+				break;
+			case B12_446:
+				/* x = -(2^75 - 2^73 + 2^63 + 2^57 + 2^50 + 2^17 + 1). */
+				bn_set_2b(t0, 75);
+				bn_set_bit(t0, 63, 1);
+				bn_set_bit(t0, 57, 1);
+				bn_set_bit(t0, 50, 1);
+				bn_set_bit(t0, 17, 1);
+				bn_add_dig(t0, t0, 1);
+				bn_set_2b(t1, 73);
+				bn_sub(t0, t0, t1);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_B12);
+				break;
 #elif FP_PRIME == 455
 			case B12_455:
-				fp_param_get_var(t0);
-				/* p = (x^2 - 2x + 1) * (x^4 - x^2 + 1)/3 + x. */
-				bn_sqr(t1, t0);
-				bn_sqr(p, t1);
-				bn_sub(p, p, t1);
-				bn_add_dig(p, p, 1);
-				bn_sub(t1, t1, t0);
-				bn_sub(t1, t1, t0);
-				bn_add_dig(t1, t1, 1);
-				bn_mul(p, p, t1);
-				bn_div_dig(p, p, 3);
-				bn_add(p, p, t0);
-				fp_prime_set_dense(p);
+				/* x = 2^76 + 2^53 + 2^31 + 2^11. */
+				bn_set_2b(t0, 76);
+				bn_set_bit(t0, 53, 1);
+				bn_set_bit(t0, 31, 1);
+				bn_set_bit(t0, 11, 1);
+				fp_prime_set_pairf(t0, EP_B12);
 				break;
 #elif FP_PRIME == 477
 			case B24_477:
-				fp_param_get_var(t0);
+				/* x = -2^48 + 2^45 + 2^31 - 2^7. */
+				bn_set_2b(t0, 48);
+				bn_set_2b(t1, 45);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 31);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 7);
+				bn_add(t0, t0, t1);
+				bn_neg(t0, t0);
 				/* p = (u - 1)^2 * (u^8 - u^4 + 1) div 3 + u. */
 				bn_sub_dig(p, t0, 1);
 				bn_sqr(p, p);
@@ -639,7 +326,15 @@ void fp_param_set(int param) {
 				break;
 #elif FP_PRIME == 508
 			case KSS_508:
-				fp_param_get_var(t0);
+				/* x = -(2^64 + 2^51 - 2^46 - 2^12). */
+				bn_set_2b(t0, 64);
+				bn_set_2b(t1, 51);
+				bn_add(t0, t0, t1);
+				bn_set_2b(t1, 46);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 12);
+				bn_sub(t0, t0, t1);
+				bn_neg(t0, t0);
 				/* h = (49*u^2 + 245 * u + 343)/3 */
 				bn_mul_dig(p, t0, 245);
 				bn_add_dig(p, p, 200);
@@ -670,6 +365,12 @@ void fp_param_set(int param) {
 				fp_prime_set_dense(p);
 				break;
 #elif FP_PRIME == 511
+			case OT_511:
+				bn_set_2b(t0, 52);
+				bn_add_dig(t0, t0, 0xAB);
+				bn_lsh(t0, t0, 12);
+				fp_prime_set_pairf(t0, EP_OT8);
+				break;
 			case PRIME_511187:
 				bn_set_2b(p, 511);
 				bn_sub_dig(p, p, 187);
@@ -682,49 +383,80 @@ void fp_param_set(int param) {
 				f[1] = 521;
 				fp_prime_set_pmers(f, 2);
 				break;
-#elif FP_PRIME == 638
-			case BN_638:
-				fp_param_get_var(t0);
-				/* p = 36 * x^4 + 36 * x^3 + 24 * x^2 + 6 * x + 1. */
-				bn_set_dig(p, 1);
-				bn_mul_dig(t1, t0, 6);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 24);
-				bn_add(p, p, t1);
-				bn_mul(t1, t0, t0);
-				bn_mul(t1, t1, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
-				bn_mul(t0, t0, t0);
-				bn_mul(t1, t0, t0);
-				bn_mul_dig(t1, t1, 36);
-				bn_add(p, p, t1);
+#elif FP_PRIME == 544
+			case CP8_544:
+				bn_read_str(p, STR_P544, strlen(STR_P544), 16);
+				/* T = 2^64 - 2^54 + 2^37 + 2^32 - 4 */
+				bn_set_2b(t0, 64);
+				bn_set_2b(t1, 54);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 37);
+				bn_add(t0, t0, t1);
+				bn_set_2b(t1, 32);
+				bn_add(t0, t0, t1);
+				bn_sub_dig(t0, t0, 4);
+				fp_prime_set_pairf(t0, EP_CP8);
 				fp_prime_set_dense(p);
 				break;
+#elif FP_PRIME == 569
+			case K54_569:
+				/* 2^27 + 2^26 + 2^22 + 2^14 + 2^6 + 2 */
+				bn_set_2b(t0, 27);
+				bn_set_2b(t1, 26);
+				bn_add(t0, t0, t1);
+				bn_set_2b(t1, 22);
+				bn_add(t0, t0, t1);
+				bn_set_2b(t1, 14);
+				bn_add(t0, t0, t1);
+				bn_add_dig(t0, t0, 66);
+				fp_prime_set_pairf(t0, EP_K54);
+				break;
+#elif FP_PRIME == 575
+			case B48_575:
+				/* x = 2^32 - 2^18 - 2^10 - 2^4. */
+				bn_set_2b(t0, 32);
+				bn_set_2b(t1, 18);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 10);
+				bn_sub(t0, t0, t1);
+				bn_sub_dig(t0, t0, 16);
+				fp_prime_set_pairf(t0, EP_B48);
+				break;
+#elif FP_PRIME == 638
+			case BN_638:
+				/* x = 2^158 - 2^128 - 2^68 + 1. */
+				bn_set_2b(t0, 158);
+				bn_set_2b(t1, 128);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 68);
+				bn_sub(t0, t0, t1);
+				bn_add_dig(t0, t0, 1);
+				fp_prime_set_pairf(t0, EP_BN);
+				break;
 			case B12_638:
-				fp_param_get_var(t0);
-				/* p = (x^2 - 2x + 1) * (x^4 - x^2 + 1)/3 + x. */
-				bn_sqr(t1, t0);
-				bn_sqr(p, t1);
-				bn_sub(p, p, t1);
-				bn_add_dig(p, p, 1);
-				bn_sub(t1, t1, t0);
-				bn_sub(t1, t1, t0);
-				bn_add_dig(t1, t1, 1);
-				bn_mul(p, p, t1);
-				bn_div_dig(p, p, 3);
-				bn_add(p, p, t0);
-				fp_prime_set_dense(p);
+				/* x = -2^107 + 2^105 + 2^93 + 2^5. */
+				bn_set_2b(t0, 107);
+				bn_set_2b(t1, 105);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 93);
+				bn_sub(t0, t0, t1);
+				bn_set_2b(t1, 5);
+				bn_sub(t0, t0, t1);
+				bn_neg(t0, t0);
+				fp_prime_set_pairf(t0, EP_B12);
 				break;
 #elif FP_PRIME == 1536
 			case SS_1536:
-				fp_param_get_var(t0);
-				bn_read_str(p, SS_P1536, strlen(SS_P1536), 16);
+				/* x = 2^255 + 2^41 + 1. */
+				bn_set_2b(t0, 255);
+				bn_set_bit(t0, 41, 1);
+				bn_add_dig(t0, t0, 1);
+				bn_read_str(p, STR_P1536, strlen(STR_P1536), 16);
 				bn_mul(p, p, t0);
 				bn_dbl(p, p);
 				bn_sub_dig(p, p, 1);
 				fp_prime_set_dense(p);
+				fp_prime_set_pairf(t0, EP_SS2);
 				break;
 #else
 			default:
@@ -778,6 +510,12 @@ int fp_param_set_any(void) {
 	fp_param_set(PRIME_383187);
 #elif FP_PRIME == 384
 	fp_param_set(NIST_384);
+#elif FP_PRIME == 446
+#ifdef FP_QNRES
+	fp_param_set(B12_446);
+#else
+	fp_param_set(BN_446);
+#endif
 #elif FP_PRIME == 455
 	fp_param_set(B12_455);
 #elif FP_PRIME == 477
@@ -785,22 +523,32 @@ int fp_param_set_any(void) {
 #elif FP_PRIME == 508
 	fp_param_set(KSS_508);
 #elif FP_PRIME == 511
-	fp_param_set(PRIME_511187);
+	fp_param_set(OT_511);
+#elif FP_PRIME == 544
+	fp_param_set(CP8_544);
+#elif FP_PRIME == 569
+	fp_param_set(K54_569);
 #elif FP_PRIME == 521
 	fp_param_set(NIST_521);
+#elif FP_PRIME == 575
+	fp_param_set(B48_575);
 #elif FP_PRIME == 638
+#ifdef FP_QNRES
 	fp_param_set(B12_638);
+#else
+	fp_param_set(BN_638);
+#endif
 #elif FP_PRIME == 1536
 	fp_param_set(SS_1536);
 #else
 	return fp_param_set_any_dense();
 #endif
-	return STS_OK;
+	return RLC_OK;
 }
 
 int fp_param_set_any_dense(void) {
 	bn_t p;
-	int result = STS_OK;
+	int result = RLC_OK;
 
 	bn_null(p);
 
@@ -808,13 +556,13 @@ int fp_param_set_any_dense(void) {
 		bn_new(p);
 #ifdef FP_QNRES
 		do {
-			bn_gen_prime(p, FP_BITS);
+			bn_gen_prime(p, RLC_FP_BITS);
 		} while ((p->dp[0] & 0x7) != 3);
 #else
-		bn_gen_prime(p, FP_BITS);
+		bn_gen_prime(p, RLC_FP_BITS);
 #endif
 		if (!bn_is_prime(p)) {
-			result = STS_ERR;
+			result = RLC_ERR;
 		} else {
 			fp_prime_set_dense(p);
 		}
@@ -842,9 +590,9 @@ int fp_param_set_any_pmers(void) {
 #elif FP_PRIME == 521
 	fp_param_set(NIST_521);
 #else
-	return STS_ERR;
+	return RLC_ERR;
 #endif
-	return STS_OK;
+	return RLC_OK;
 }
 
 int fp_param_set_any_tower(void) {
@@ -858,14 +606,32 @@ int fp_param_set_any_tower(void) {
 	fp_param_set(B12_381);
 #elif FP_PRIME == 382
 	fp_param_set(BN_382);
+#elif FP_PRIME == 446
+#ifdef FP_QNRES
+	fp_param_set(B12_446);
+#else
+	fp_param_set(BN_446);
+#endif
 #elif FP_PRIME == 455
 	fp_param_set(B12_455);
 #elif FP_PRIME == 477
 	fp_param_set(B24_477);
 #elif FP_PRIME == 508
 	fp_param_set(KSS_508);
+#elif FP_PRIME == 511
+	fp_param_set(OT_511);
+#elif FP_PRIME == 544
+	fp_param_set(CP8_544);
+#elif FP_PRIME == 569
+	fp_param_set(K54_569);
+#elif FP_PRIME == 575
+	fp_param_set(B48_575);
 #elif FP_PRIME == 638
+#ifdef FP_QNRES
 	fp_param_set(B12_638);
+#else
+	fp_param_set(BN_638);
+#endif
 #elif FP_PRIME == 1536
 	fp_param_set(SS_1536);
 #else
@@ -875,7 +641,7 @@ int fp_param_set_any_tower(void) {
 	} while (fp_prime_get_mod8() == 1 || fp_prime_get_mod8() == 5);
 #endif
 
-	return STS_OK;
+	return RLC_OK;
 }
 
 void fp_param_print(void) {
