@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -35,7 +36,7 @@
 
 static int memory(void) {
 	err_t e;
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a;
 
 	ep_null(a);
@@ -54,15 +55,15 @@ static int memory(void) {
 		}
 	}
 	(void)a;
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	return code;
 }
 
 int util(void) {
-	int l, code = STS_ERR;
+	int l, code = RLC_ERR;
 	ep_t a, b, c;
-	uint8_t bin[2 * FP_BYTES + 1];
+	uint8_t bin[2 * RLC_FP_BYTES + 1];
 
 	ep_null(a);
 	ep_null(b);
@@ -78,40 +79,40 @@ int util(void) {
 			ep_rand(b);
 			ep_rand(c);
 			/* Compare points in affine coordinates. */
-			if (ep_cmp(a, c) != CMP_EQ) {
+			if (ep_cmp(a, c) != RLC_EQ) {
 				ep_copy(c, a);
-				TEST_ASSERT(ep_cmp(c, a) == CMP_EQ, end);
+				TEST_ASSERT(ep_cmp(c, a) == RLC_EQ, end);
 			}
-			if (ep_cmp(b, c) != CMP_EQ) {
+			if (ep_cmp(b, c) != RLC_EQ) {
 				ep_copy(c, b);
-				TEST_ASSERT(ep_cmp(b, c) == CMP_EQ, end);
+				TEST_ASSERT(ep_cmp(b, c) == RLC_EQ, end);
 			}
 			/* Compare with one point in projective. */
 			ep_dbl(c, a);
 			ep_norm(c, c);
 			ep_dbl(a, a);
-			TEST_ASSERT(ep_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(ep_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(ep_cmp(a, c) == RLC_EQ, end);
 			/* Compare with two points in projective. */
 			ep_dbl(c, c);
 			ep_dbl(a, a);
-			TEST_ASSERT(ep_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(ep_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(ep_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 
 		TEST_BEGIN("negation and comparison are consistent") {
 			ep_rand(a);
 			ep_neg(b, a);
-			TEST_ASSERT(ep_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
 		TEST_BEGIN("assignment to random and comparison are consistent") {
 			ep_rand(a);
 			ep_set_infty(c);
-			TEST_ASSERT(ep_cmp(a, c) != CMP_EQ, end);
-			TEST_ASSERT(ep_cmp(c, a) != CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(a, c) != RLC_EQ, end);
+			TEST_ASSERT(ep_cmp(c, a) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -122,6 +123,8 @@ int util(void) {
 		TEST_END;
 
 		TEST_BEGIN("validity test is correct") {
+			ep_set_infty(a);
+			TEST_ASSERT(ep_is_valid(a), end);
 			ep_rand(a);
 			TEST_ASSERT(ep_is_valid(a), end);
 			fp_rand(a->x);
@@ -135,19 +138,19 @@ int util(void) {
 				l = ep_size_bin(a, j);
 				ep_write_bin(bin, l, a, j);
 				ep_read_bin(b, bin, l);
-				TEST_ASSERT(ep_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep_cmp(a, b) == RLC_EQ, end);
 				ep_rand(a);
 				l = ep_size_bin(a, j);
 				ep_write_bin(bin, l, a, j);
 				ep_read_bin(b, bin, l);
-				TEST_ASSERT(ep_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep_cmp(a, b) == RLC_EQ, end);
 				ep_rand(a);
 				ep_dbl(a, a);
 				l = ep_size_bin(a, j);
 				ep_norm(a, a);
 				ep_write_bin(bin, l, a, j);
 				ep_read_bin(b, bin, l);
-				TEST_ASSERT(ep_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep_cmp(a, b) == RLC_EQ, end);
 			}
 		}
 		TEST_END;
@@ -156,7 +159,7 @@ int util(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	ep_free(b);
@@ -165,7 +168,7 @@ int util(void) {
 }
 
 int addition(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a, b, c, d, e;
 
 	ep_null(a);
@@ -186,7 +189,9 @@ int addition(void) {
 			ep_rand(b);
 			ep_add(d, a, b);
 			ep_add(e, b, a);
-			TEST_ASSERT(ep_cmp(d, e) == CMP_EQ, end);
+			ep_norm(d, d);
+			ep_norm(e, e);
+			TEST_ASSERT(ep_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition is associative") {
@@ -197,16 +202,16 @@ int addition(void) {
 			ep_add(d, d, c);
 			ep_add(e, b, c);
 			ep_add(e, e, a);
-			TEST_ASSERT(ep_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has identity") {
 			ep_rand(a);
 			ep_set_infty(d);
 			ep_add(e, a, d);
-			TEST_ASSERT(ep_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, a) == RLC_EQ, end);
 			ep_add(e, d, a);
-			TEST_ASSERT(ep_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, a) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has inverse") {
@@ -223,7 +228,7 @@ int addition(void) {
 			ep_add(d, a, b);
 			ep_norm(d, d);
 			ep_add_basic(e, a, b);
-			TEST_ASSERT(ep_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -243,7 +248,7 @@ int addition(void) {
 			ep_norm(b, b);
 			ep_add(e, a, b);
 			ep_norm(e, e);
-			TEST_ASSERT(ep_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -259,7 +264,7 @@ int addition(void) {
 			ep_norm(a, a);
 			ep_add(e, a, b);
 			ep_norm(e, e);
-			TEST_ASSERT(ep_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition in mixed coordinates (z1,z2 = 1) is correct") {
@@ -272,7 +277,7 @@ int addition(void) {
 			ep_norm(d, d);
 			ep_add_projc(e, a, b);
 			ep_norm(e, e);
-			TEST_ASSERT(ep_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -280,7 +285,7 @@ int addition(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	ep_free(b);
@@ -291,7 +296,7 @@ int addition(void) {
 }
 
 int subtraction(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a, b, c, d;
 
 	ep_null(a);
@@ -311,7 +316,7 @@ int subtraction(void) {
 			ep_sub(c, a, b);
 			ep_sub(d, b, a);
 			ep_neg(d, d);
-			TEST_ASSERT(ep_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, d) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -319,7 +324,7 @@ int subtraction(void) {
 			ep_rand(a);
 			ep_set_infty(c);
 			ep_sub(d, a, c);
-			TEST_ASSERT(ep_cmp(d, a) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(d, a) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -337,7 +342,7 @@ int subtraction(void) {
 			ep_sub(c, a, b);
 			ep_norm(c, c);
 			ep_sub_basic(d, a, b);
-			TEST_ASSERT(ep_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -357,7 +362,7 @@ int subtraction(void) {
 			ep_norm(b, b);
 			ep_sub(d, a, b);
 			ep_norm(d, d);
-			TEST_ASSERT(ep_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -373,7 +378,7 @@ int subtraction(void) {
 			ep_norm(a, a);
 			ep_sub(d, a, b);
 			ep_norm(d, d);
-			TEST_ASSERT(ep_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN
@@ -388,14 +393,14 @@ int subtraction(void) {
 			ep_norm(c, c);
 			ep_sub_projc(d, a, b);
 			ep_norm(d, d);
-			TEST_ASSERT(ep_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	ep_free(b);
@@ -405,7 +410,7 @@ int subtraction(void) {
 }
 
 int doubling(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a, b, c;
 
 	ep_null(a);
@@ -421,7 +426,7 @@ int doubling(void) {
 			ep_rand(a);
 			ep_add(b, a, a);
 			ep_dbl(c, a);
-			TEST_ASSERT(ep_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_ADD == BASIC || !defined(STRIP)
@@ -430,7 +435,7 @@ int doubling(void) {
 			ep_dbl(b, a);
 			ep_norm(b, b);
 			ep_dbl_basic(c, a);
-			TEST_ASSERT(ep_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -444,7 +449,7 @@ int doubling(void) {
 			ep_norm(a, a);
 			ep_dbl(c, a);
 			ep_norm(c, c);
-			TEST_ASSERT(ep_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point doubling in mixed coordinates (z1 = 1) is correct") {
@@ -453,14 +458,14 @@ int doubling(void) {
 			ep_norm(b, b);
 			ep_dbl(c, a);
 			ep_norm(c, c);
-			TEST_ASSERT(ep_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 #endif
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	ep_free(b);
@@ -469,7 +474,7 @@ int doubling(void) {
 }
 
 static int multiplication(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k;
 	ep_t p, q, r;
 
@@ -490,6 +495,7 @@ static int multiplication(void) {
 		ep_curve_get_ord(n);
 
 		TEST_BEGIN("generator has the right order") {
+			TEST_ASSERT(ep_is_valid(p), end);
 			ep_mul(r, p, n);
 			TEST_ASSERT(ep_is_infty(r) == 1, end);
 		} TEST_END;
@@ -500,15 +506,15 @@ static int multiplication(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_gen(r, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_gen(r, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_gen(r, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_MUL == BASIC || !defined(STRIP)
@@ -518,18 +524,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_basic(r, p, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			ep_rand(p);
 			ep_mul(r, p, n);
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_basic(r, p, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_basic(r, p, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -540,18 +546,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_slide(r, p, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			ep_rand(p);
 			ep_mul(r, p, n);
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_slide(r, p, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_slide(r, p, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -563,18 +569,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_monty(r, p, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			ep_rand(p);
 			ep_mul(r, p, n);
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_monty(r, p, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_monty(r, p, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -586,18 +592,41 @@ static int multiplication(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_lwnaf(r, p, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			ep_rand(p);
 			ep_mul(r, p, n);
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_lwnaf(r, p, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_lwnaf(r, p, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
+		}
+		TEST_END;
+#endif
+
+#if EP_MUL == LWREG || !defined(STRIP)
+		TEST_BEGIN("left-to-right regular point multiplication is correct") {
+			bn_zero(k);
+			ep_mul_lwreg(r, p, k);
+			TEST_ASSERT(ep_is_infty(r), end);
+			bn_set_dig(k, 1);
+			ep_mul_lwreg(r, p, k);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
+			ep_rand(p);
+			ep_mul(r, p, n);
+			TEST_ASSERT(ep_is_infty(r), end);
+			bn_rand_mod(k, n);
+			ep_mul(q, p, k);
+			ep_mul_lwreg(r, p, k);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
+			bn_neg(k, k);
+			ep_mul_lwreg(r, p, k);
+			ep_neg(r, r);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -606,11 +635,11 @@ static int multiplication(void) {
 			ep_mul_dig(r, p, 0);
 			TEST_ASSERT(ep_is_infty(r), end);
 			ep_mul_dig(r, p, 1);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
-			bn_rand(k, BN_POS, BN_DIGIT);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
+			bn_rand(k, RLC_POS, RLC_DIG);
 			ep_mul(q, p, k);
 			ep_mul_dig(r, p, k->dp[0]);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 	}
@@ -618,7 +647,7 @@ static int multiplication(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -629,9 +658,9 @@ static int multiplication(void) {
 }
 
 static int fixed(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k;
-	ep_t p, q, r, t[RELIC_EP_TABLE_MAX];
+	ep_t p, q, r, t[RLC_EP_TABLE_MAX];
 
 	bn_null(n);
 	bn_null(k);
@@ -639,7 +668,7 @@ static int fixed(void) {
 	ep_null(q);
 	ep_null(r);
 
-	for (int i = 0; i < RELIC_EP_TABLE_MAX; i++) {
+	for (int i = 0; i < RLC_EP_TABLE_MAX; i++) {
 		ep_null(t[i]);
 	}
 
@@ -653,7 +682,7 @@ static int fixed(void) {
 		ep_curve_get_gen(p);
 		ep_curve_get_ord(n);
 
-		for (int i = 0; i < RELIC_EP_TABLE; i++) {
+		for (int i = 0; i < RLC_EP_TABLE; i++) {
 			ep_new(t[i]);
 		}
 		TEST_BEGIN("fixed point multiplication is correct") {
@@ -664,23 +693,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_fix(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(q, p, k);
 			ep_mul_fix(q, (const ep_t *)t, k);
 			ep_mul(r, p, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_fix(r, (const ep_t *)t, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE; i++) {
+		for (int i = 0; i < RLC_EP_TABLE; i++) {
 			ep_free(t[i]);
 		}
 
 #if EP_FIX == BASIC || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_BASIC; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_BASIC; i++) {
 			ep_new(t[i]);
 		}
 		TEST_BEGIN("binary fixed point multiplication is correct") {
@@ -691,77 +720,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_fix_basic(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(r, p, k);
 			ep_mul_fix_basic(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_fix_basic(r, (const ep_t *)t, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_BASIC; i++) {
-			ep_free(t[i]);
-		}
-#endif
-
-#if EP_FIX == YAOWI || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_YAOWI; i++) {
-			ep_new(t[i]);
-		}
-		TEST_BEGIN("yao windowing fixed point multiplication is correct") {
-			ep_rand(p);
-			ep_mul_pre_yaowi(t, p);
-			bn_zero(k);
-			ep_mul_fix_yaowi(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_is_infty(r), end);
-			bn_set_dig(k, 1);
-			ep_mul_fix_yaowi(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
-			bn_rand_mod(k, n);
-			ep_mul(r, p, k);
-			ep_mul_fix_yaowi(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
-			bn_neg(k, k);
-			ep_mul_fix_yaowi(r, (const ep_t *)t, k);
-			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
-		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_YAOWI; i++) {
-			ep_free(t[i]);
-		}
-#endif
-
-#if EP_FIX == NAFWI || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_NAFWI; i++) {
-			ep_new(t[i]);
-		}
-		TEST_BEGIN("naf windowing fixed point multiplication is correct") {
-			ep_rand(p);
-			ep_mul_pre_nafwi(t, p);
-			bn_zero(k);
-			ep_mul_fix_nafwi(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_is_infty(r), end);
-			bn_set_dig(k, 1);
-			ep_mul_fix_nafwi(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
-			bn_rand_mod(k, n);
-			ep_mul(r, p, k);
-			ep_mul_fix_nafwi(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
-			bn_neg(k, k);
-			ep_mul_fix_nafwi(r, (const ep_t *)t, k);
-			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
-		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_NAFWI; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_BASIC; i++) {
 			ep_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == COMBS || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_COMBS; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBS; i++) {
 			ep_new(t[i]);
 		}
 		TEST_BEGIN("single-table comb fixed point multiplication is correct") {
@@ -772,23 +747,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_fix_combs(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(r, p, k);
 			ep_mul_fix_combs(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_fix_combs(r, (const ep_t *)t, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_COMBS; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBS; i++) {
 			ep_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == COMBD || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_COMBD; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBD; i++) {
 			ep_new(t[i]);
 		}
 		TEST_BEGIN("double-table comb fixed point multiplication is correct") {
@@ -799,23 +774,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_fix_combd(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(r, p, k);
 			ep_mul_fix_combd(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_fix_combd(r, (const ep_t *)t, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_COMBD; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBD; i++) {
 			ep_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == LWNAF || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_LWNAF; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_LWNAF; i++) {
 			ep_new(t[i]);
 		}
 		TEST_BEGIN("left-to-right w-naf fixed point multiplication is correct") {
@@ -826,17 +801,17 @@ static int fixed(void) {
 			TEST_ASSERT(ep_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep_mul_fix_lwnaf(r, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep_mul(r, p, k);
 			ep_mul_fix_lwnaf(q, (const ep_t *)t, k);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_fix_lwnaf(r, (const ep_t *)t, k);
 			ep_neg(r, r);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_LWNAF; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_LWNAF; i++) {
 			ep_free(t[i]);
 		}
 #endif
@@ -845,7 +820,7 @@ static int fixed(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(p);
 	ep_free(q);
@@ -856,7 +831,7 @@ static int fixed(void) {
 }
 
 static int simultaneous(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k, l;
 	ep_t p, q, r;
 
@@ -883,31 +858,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul(q, p, k);
 			ep_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_SIM == BASIC || !defined(STRIP)
@@ -916,31 +893,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim_basic(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul(q, p, k);
 			ep_mul_sim_basic(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim_basic(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim_basic(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim_basic(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -950,31 +929,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim_trick(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul(q, p, k);
 			ep_mul_sim_trick(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim_trick(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim_trick(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim_trick(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -984,31 +965,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim_inter(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul(q, p, k);
 			ep_mul_sim_inter(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim_inter(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim_inter(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim_inter(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -1018,31 +1001,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim_joint(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul(q, p, k);
 			ep_mul_sim_joint(r, p, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim_joint(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim_joint(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			ep_norm(q, q);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim_joint(r, p, k, q, l);
 			ep_mul(p, p, k);
 			ep_mul(q, q, l);
 			ep_add(q, q, p);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -1051,35 +1036,35 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep_mul(q, p, l);
 			ep_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep_mul_gen(q, k);
 			ep_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep_mul_sim_gen(r, k, q, l);
 			ep_curve_get_gen(p);
 			ep_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep_mul_sim_gen(r, k, q, l);
 			ep_curve_get_gen(p);
 			ep_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep_mul_sim_gen(r, k, q, l);
 			ep_curve_get_gen(p);
 			ep_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -1091,7 +1076,7 @@ static int simultaneous(void) {
 }
 
 static int compression(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a, b, c;
 
 	ep_null(a);
@@ -1107,14 +1092,14 @@ static int compression(void) {
 			ep_rand(a);
 			ep_pck(b, a);
 			TEST_ASSERT(ep_upk(c, b) == 1, end);
-			TEST_ASSERT(ep_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	ep_free(b);
@@ -1123,7 +1108,7 @@ static int compression(void) {
 }
 
 static int hashing(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep_t a;
 	bn_t n;
 	uint8_t msg[5];
@@ -1140,6 +1125,7 @@ static int hashing(void) {
 		TEST_BEGIN("point hashing is correct") {
 			rand_bytes(msg, sizeof(msg));
 			ep_map(a, msg, sizeof(msg));
+			TEST_ASSERT(ep_is_infty(a) == 0, end);
 			ep_mul(a, a, n);
 			TEST_ASSERT(ep_is_infty(a) == 1, end);
 		}
@@ -1148,7 +1134,7 @@ static int hashing(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep_free(a);
 	bn_free(n);
@@ -1160,56 +1146,56 @@ int test(void) {
 
 	util_banner("Utilities:", 1);
 
-	if (memory() != STS_OK) {
+	if (memory() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (util() != STS_OK) {
-		return STS_ERR;
+	if (util() != RLC_OK) {
+		return RLC_ERR;
 	}
 
 	util_banner("Arithmetic:", 1);
 
-	if (addition() != STS_OK) {
-		return STS_ERR;
+	if (addition() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (subtraction() != STS_OK) {
-		return STS_ERR;
+	if (subtraction() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (doubling() != STS_OK) {
-		return STS_ERR;
+	if (doubling() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (multiplication() != STS_OK) {
-		return STS_ERR;
+	if (multiplication() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (fixed() != STS_OK) {
-		return STS_ERR;
+	if (fixed() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (simultaneous() != STS_OK) {
-		return STS_ERR;
+	if (simultaneous() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (compression() != STS_OK) {
-		return STS_ERR;
+	if (compression() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (hashing() != STS_OK) {
-		return STS_ERR;
+	if (hashing() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	return STS_OK;
+	return RLC_OK;
 }
 
 int main(void) {
-	int r0 = STS_ERR, r1 = STS_ERR, r2 = STS_ERR, r3 = STS_ERR;
+	int r0 = RLC_ERR, r1 = RLC_ERR, r2 = RLC_ERR, r3 = RLC_ERR;
 
-	if (core_init() != STS_OK) {
+	if (core_init() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
@@ -1218,8 +1204,8 @@ int main(void) {
 
 #if defined(EP_PLAIN)
 	r0 = ep_param_set_any_plain();
-	if (r0 == STS_OK) {
-		if (test() != STS_OK) {
+	if (r0 == RLC_OK) {
+		if (test() != RLC_OK) {
 			core_clean();
 			return 1;
 		}
@@ -1228,8 +1214,8 @@ int main(void) {
 
 #if defined(EP_ENDOM)
 	r1 = ep_param_set_any_endom();
-	if (r1 == STS_OK) {
-		if (test() != STS_OK) {
+	if (r1 == RLC_OK) {
+		if (test() != RLC_OK) {
 			core_clean();
 			return 1;
 		}
@@ -1237,8 +1223,8 @@ int main(void) {
 #endif
 
 	r2 = ep_param_set_any_pairf();
-	if (r2 == STS_OK) {
-		if (test() != STS_OK) {
+	if (r2 == RLC_OK) {
+		if (test() != RLC_OK) {
 			core_clean();
 			return 1;
 		}
@@ -1246,21 +1232,21 @@ int main(void) {
 
 #if defined(EP_SUPER)
 	r3 = ep_param_set_any_super();
-	if (r3 == STS_OK) {
-		if (test() != STS_OK) {
+	if (r3 == RLC_OK) {
+		if (test() != RLC_OK) {
 			core_clean();
 			return 1;
 		}
 	}
 #endif
 
-	if (r0 == STS_ERR && r1 == STS_ERR && r2 == STS_ERR && r3 == STS_ERR) {
-		if (ep_param_set_any() == STS_ERR) {
+	if (r0 == RLC_ERR && r1 == RLC_ERR && r2 == RLC_ERR && r3 == RLC_ERR) {
+		if (ep_param_set_any() == RLC_ERR) {
 			THROW(ERR_NO_CURVE);
 			core_clean();
 			return 0;
 		} else {
-			if (test() != STS_OK) {
+			if (test() != RLC_OK) {
 				core_clean();
 				return 1;
 			}
