@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -35,7 +36,7 @@
 
 static int memory(void) {
 	err_t e;
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a;
 
 	ep2_null(a);
@@ -54,16 +55,16 @@ static int memory(void) {
 		}
 	}
 	(void)a;
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	return code;
 }
 
 int util(void) {
-	int l, code = STS_ERR;
+	int l, code = RLC_ERR;
 	ep2_t a, b, c;
 	bn_t n;
-	uint8_t bin[4 * FP_BYTES + 1];
+	uint8_t bin[4 * RLC_FP_BYTES + 1];
 
 	ep2_null(a);
 	ep2_null(b);
@@ -79,7 +80,7 @@ int util(void) {
 		TEST_BEGIN("comparison is consistent") {
 			ep2_rand(a);
 			ep2_rand(b);
-			TEST_ASSERT(ep2_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -88,32 +89,32 @@ int util(void) {
 			ep2_rand(b);
 			ep2_rand(c);
 			/* Compare points in affine coordinates. */
-			if (ep2_cmp(a, c) != CMP_EQ) {
+			if (ep2_cmp(a, c) != RLC_EQ) {
 				ep2_copy(c, a);
-				TEST_ASSERT(ep2_cmp(c, a) == CMP_EQ, end);
+				TEST_ASSERT(ep2_cmp(c, a) == RLC_EQ, end);
 			}
-			if (ep2_cmp(b, c) != CMP_EQ) {
+			if (ep2_cmp(b, c) != RLC_EQ) {
 				ep2_copy(c, b);
-				TEST_ASSERT(ep2_cmp(b, c) == CMP_EQ, end);
+				TEST_ASSERT(ep2_cmp(b, c) == RLC_EQ, end);
 			}
 			/* Compare with one point in projective. */
 			ep2_dbl(c, a);
 			ep2_norm(c, c);
 			ep2_dbl(a, a);
-			TEST_ASSERT(ep2_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(ep2_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, c) == RLC_EQ, end);
 			/* Compare with two points in projective. */
 			ep2_dbl(c, c);
 			ep2_dbl(a, a);
-			TEST_ASSERT(ep2_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(ep2_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 
 		TEST_BEGIN("negation and comparison are consistent") {
 			ep2_rand(a);
 			ep2_neg(b, a);
-			TEST_ASSERT(ep2_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -122,8 +123,8 @@ int util(void) {
 		{
 			ep2_rand(a);
 			ep2_set_infty(c);
-			TEST_ASSERT(ep2_cmp(a, c) != CMP_EQ, end);
-			TEST_ASSERT(ep2_cmp(c, a) != CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, c) != RLC_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, a) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -134,6 +135,8 @@ int util(void) {
 		TEST_END;
 
 		TEST_BEGIN("validity test is correct") {
+			ep2_set_infty(a);
+			TEST_ASSERT(ep2_is_valid(a), end);
 			ep2_rand(a);
 			TEST_ASSERT(ep2_is_valid(a), end);
 			fp2_rand(a->x);
@@ -147,19 +150,19 @@ int util(void) {
 				l = ep2_size_bin(a, j);
 				ep2_write_bin(bin, l, a, j);
 				ep2_read_bin(b, bin, l);
-				TEST_ASSERT(ep2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep2_cmp(a, b) == RLC_EQ, end);
 				ep2_rand(a);
 				l = ep2_size_bin(a, j);
 				ep2_write_bin(bin, l, a, j);
 				ep2_read_bin(b, bin, l);
-				TEST_ASSERT(ep2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep2_cmp(a, b) == RLC_EQ, end);
 				ep2_rand(a);
 				ep2_dbl(a, a);
 				l = ep2_size_bin(a, j);
 				ep2_norm(a, a);
 				ep2_write_bin(bin, l, a, j);
 				ep2_read_bin(b, bin, l);
-				TEST_ASSERT(ep2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(ep2_cmp(a, b) == RLC_EQ, end);
 			}
 		}
 		TEST_END;
@@ -168,7 +171,7 @@ int util(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -178,7 +181,7 @@ int util(void) {
 }
 
 int addition(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a, b, c, d, e;
 
 	ep2_null(a);
@@ -199,7 +202,7 @@ int addition(void) {
 			ep2_rand(b);
 			ep2_add(d, a, b);
 			ep2_add(e, b, a);
-			TEST_ASSERT(ep2_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition is associative") {
@@ -210,16 +213,16 @@ int addition(void) {
 			ep2_add(d, d, c);
 			ep2_add(e, b, c);
 			ep2_add(e, e, a);
-			TEST_ASSERT(ep2_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has identity") {
 			ep2_rand(a);
 			ep2_set_infty(d);
 			ep2_add(e, a, d);
-			TEST_ASSERT(ep2_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, a) == RLC_EQ, end);
 			ep2_add(e, d, a);
-			TEST_ASSERT(ep2_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, a) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has inverse") {
@@ -236,7 +239,7 @@ int addition(void) {
 			ep2_add(d, a, b);
 			ep2_norm(d, d);
 			ep2_add_basic(e, a, b);
-			TEST_ASSERT(ep2_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -257,7 +260,7 @@ int addition(void) {
 			ep2_norm(b, b);
 			ep2_add(e, a, b);
 			ep2_norm(e, e);
-			TEST_ASSERT(ep2_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -273,7 +276,7 @@ int addition(void) {
 			ep2_norm(a, a);
 			ep2_add(e, a, b);
 			ep2_norm(e, e);
-			TEST_ASSERT(ep2_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition in mixed coordinates (z1,z2 = 1) is correct") {
@@ -286,7 +289,7 @@ int addition(void) {
 			ep2_norm(d, d);
 			ep2_add_projc(e, a, b);
 			ep2_norm(e, e);
-			TEST_ASSERT(ep2_cmp(e, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(e, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -294,7 +297,7 @@ int addition(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -305,7 +308,7 @@ int addition(void) {
 }
 
 int subtraction(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a, b, c, d;
 
 	ep2_null(a);
@@ -325,7 +328,7 @@ int subtraction(void) {
 			ep2_sub(c, a, b);
 			ep2_sub(d, b, a);
 			ep2_neg(d, d);
-			TEST_ASSERT(ep2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, d) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -333,7 +336,7 @@ int subtraction(void) {
 			ep2_rand(a);
 			ep2_set_infty(c);
 			ep2_sub(d, a, c);
-			TEST_ASSERT(ep2_cmp(d, a) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(d, a) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -351,7 +354,7 @@ int subtraction(void) {
 			ep2_sub(c, a, b);
 			ep2_norm(c, c);
 			ep2_sub_basic(d, a, b);
-			TEST_ASSERT(ep2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -371,7 +374,7 @@ int subtraction(void) {
 			ep2_norm(b, b);
 			ep2_sub(d, a, b);
 			ep2_norm(d, d);
-			TEST_ASSERT(ep2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -387,7 +390,7 @@ int subtraction(void) {
 			ep2_norm(a, a);
 			ep2_sub(d, a, b);
 			ep2_norm(d, d);
-			TEST_ASSERT(ep2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN
@@ -402,14 +405,14 @@ int subtraction(void) {
 			ep2_norm(c, c);
 			ep2_sub_projc(d, a, b);
 			ep2_norm(d, d);
-			TEST_ASSERT(ep2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, d) == RLC_EQ, end);
 		} TEST_END;
 #endif
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -419,7 +422,7 @@ int subtraction(void) {
 }
 
 int doubling(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a, b, c;
 
 	ep2_null(a);
@@ -435,7 +438,7 @@ int doubling(void) {
 			ep2_rand(a);
 			ep2_add(b, a, a);
 			ep2_dbl(c, a);
-			TEST_ASSERT(ep2_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_ADD == BASIC || !defined(STRIP)
@@ -444,7 +447,7 @@ int doubling(void) {
 			ep2_dbl(b, a);
 			ep2_norm(b, b);
 			ep2_dbl_basic(c, a);
-			TEST_ASSERT(ep2_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -458,7 +461,7 @@ int doubling(void) {
 			ep2_norm(a, a);
 			ep2_dbl(c, a);
 			ep2_norm(c, c);
-			TEST_ASSERT(ep2_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point doubling in mixed coordinates (z1 = 1) is correct") {
@@ -467,14 +470,14 @@ int doubling(void) {
 			ep2_norm(b, b);
 			ep2_dbl(c, a);
 			ep2_norm(c, c);
-			TEST_ASSERT(ep2_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 #endif
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -483,7 +486,7 @@ int doubling(void) {
 }
 
 static int multiplication(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k;
 	ep2_t p, q, r;
 
@@ -503,21 +506,27 @@ static int multiplication(void) {
 		ep2_curve_get_gen(p);
 		ep2_curve_get_ord(n);
 
+		TEST_BEGIN("generator has the right order") {
+			TEST_ASSERT(ep2_is_valid(p), end);
+			ep2_mul(r, p, n);
+			TEST_ASSERT(ep2_is_infty(r) == 1, end);
+		} TEST_END;
+
 		TEST_BEGIN("generator multiplication is correct") {
 			bn_zero(k);
 			ep2_mul_gen(r, k);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_gen(r, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_gen(r, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_gen(r, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_MUL == BASIC || !defined(STRIP)
@@ -527,18 +536,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_basic(r, p, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			ep2_rand(p);
 			ep2_mul(r, p, n);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_basic(r, p, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_basic(r, p, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -549,18 +558,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_slide(r, p, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			ep2_rand(p);
 			ep2_mul(r, p, n);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_slide(r, p, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_slide(r, p, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -572,18 +581,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_monty(r, p, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			ep2_rand(p);
 			ep2_mul(r, p, n);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_monty(r, p, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_monty(r, p, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -595,18 +604,18 @@ static int multiplication(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_lwnaf(r, p, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			ep2_rand(p);
 			ep2_mul(r, p, n);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_lwnaf(r, p, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_lwnaf(r, p, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 #endif
@@ -615,11 +624,11 @@ static int multiplication(void) {
 			ep2_mul_dig(r, p, 0);
 			TEST_ASSERT(ep2_is_infty(r), end);
 			ep2_mul_dig(r, p, 1);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
-			bn_rand(k, BN_POS, BN_DIGIT);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
+			bn_rand(k, RLC_POS, RLC_DIG);
 			ep2_mul(q, p, k);
 			ep2_mul_dig(r, p, k->dp[0]);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		}
 		TEST_END;
 	}
@@ -627,7 +636,7 @@ static int multiplication(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -638,9 +647,9 @@ static int multiplication(void) {
 }
 
 static int fixed(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k;
-	ep2_t p, q, r, t[RELIC_EPX_TABLE_MAX];
+	ep2_t p, q, r, t[RLC_EPX_TABLE_MAX];
 
 	bn_null(n);
 	bn_null(k);
@@ -648,7 +657,7 @@ static int fixed(void) {
 	ep2_null(q);
 	ep2_null(r);
 
-	for (int i = 0; i < RELIC_EPX_TABLE_MAX; i++) {
+	for (int i = 0; i < RLC_EPX_TABLE_MAX; i++) {
 		ep2_null(t[i]);
 	}
 
@@ -662,7 +671,7 @@ static int fixed(void) {
 		ep2_curve_get_gen(p);
 		ep2_curve_get_ord(n);
 
-		for (int i = 0; i < RELIC_EP_TABLE; i++) {
+		for (int i = 0; i < RLC_EP_TABLE; i++) {
 			ep2_new(t[i]);
 		}
 		TEST_BEGIN("fixed point multiplication is correct") {
@@ -673,23 +682,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_fix(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(q, p, k);
 			ep2_mul_fix(q, t, k);
 			ep2_mul(r, p, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_fix(r, t, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE; i++) {
+		for (int i = 0; i < RLC_EP_TABLE; i++) {
 			ep2_free(t[i]);
 		}
 
 #if EP_FIX == BASIC || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_BASIC; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_BASIC; i++) {
 			ep2_new(t[i]);
 		}
 		TEST_BEGIN("binary fixed point multiplication is correct") {
@@ -700,77 +709,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_fix_basic(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(r, p, k);
 			ep2_mul_fix_basic(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_fix_basic(r, t, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_BASIC; i++) {
-			ep2_free(t[i]);
-		}
-#endif
-
-#if EP_FIX == YAOWI || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_YAOWI; i++) {
-			ep2_new(t[i]);
-		}
-		TEST_BEGIN("yao windowing fixed point multiplication is correct") {
-			ep2_rand(p);
-			ep2_mul_pre_yaowi(t, p);
-			bn_zero(k);
-			ep2_mul_fix_yaowi(r, t, k);
-			TEST_ASSERT(ep2_is_infty(r), end);
-			bn_set_dig(k, 1);
-			ep2_mul_fix_yaowi(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
-			bn_rand_mod(k, n);
-			ep2_mul(r, p, k);
-			ep2_mul_fix_yaowi(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
-			bn_neg(k, k);
-			ep2_mul_fix_yaowi(r, t, k);
-			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
-		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_YAOWI; i++) {
-			ep2_free(t[i]);
-		}
-#endif
-
-#if EP_FIX == NAFWI || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_NAFWI; i++) {
-			ep2_new(t[i]);
-		}
-		TEST_BEGIN("naf windowing fixed point multiplication is correct") {
-			ep2_rand(p);
-			ep2_mul_pre_nafwi(t, p);
-			bn_zero(k);
-			ep2_mul_fix_nafwi(r, t, k);
-			TEST_ASSERT(ep2_is_infty(r), end);
-			bn_set_dig(k, 1);
-			ep2_mul_fix_nafwi(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
-			bn_rand_mod(k, n);
-			ep2_mul(r, p, k);
-			ep2_mul_fix_nafwi(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
-			bn_neg(k, k);
-			ep2_mul_fix_nafwi(r, t, k);
-			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
-		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_NAFWI; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_BASIC; i++) {
 			ep2_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == COMBS || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_COMBS; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBS; i++) {
 			ep2_new(t[i]);
 		}
 		TEST_BEGIN("single-table comb fixed point multiplication is correct") {
@@ -781,23 +736,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_fix_combs(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(r, p, k);
 			ep2_mul_fix_combs(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_fix_combs(r, t, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_COMBS; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBS; i++) {
 			ep2_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == COMBD || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_COMBD; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBD; i++) {
 			ep2_new(t[i]);
 		}
 		TEST_BEGIN("double-table comb fixed point multiplication is correct") {
@@ -808,23 +763,23 @@ static int fixed(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_fix_combd(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(r, p, k);
 			ep2_mul_fix_combd(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_fix_combd(r, t, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_COMBD; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_COMBD; i++) {
 			ep2_free(t[i]);
 		}
 #endif
 
 #if EP_FIX == LWNAF || !defined(STRIP)
-		for (int i = 0; i < RELIC_EP_TABLE_LWNAF; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_LWNAF; i++) {
 			ep2_new(t[i]);
 		}
 		TEST_BEGIN("left-to-right w-naf fixed point multiplication is correct") {
@@ -835,17 +790,17 @@ static int fixed(void) {
 			TEST_ASSERT(ep2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			ep2_mul_fix_lwnaf(r, t, k);
-			TEST_ASSERT(ep2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			ep2_mul(r, p, k);
 			ep2_mul_fix_lwnaf(q, t, k);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_fix_lwnaf(r, t, k);
 			ep2_neg(r, r);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_EP_TABLE_LWNAF; i++) {
+		for (int i = 0; i < RLC_EP_TABLE_LWNAF; i++) {
 			ep2_free(t[i]);
 		}
 #endif
@@ -854,7 +809,7 @@ static int fixed(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(p);
 	ep2_free(q);
@@ -865,7 +820,7 @@ static int fixed(void) {
 }
 
 static int simultaneous(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n, k, l;
 	ep2_t p, q, r;
 
@@ -892,31 +847,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul(q, p, k);
 			ep2_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 #if EP_SIM == BASIC || !defined(STRIP)
@@ -925,31 +882,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim_basic(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul(q, p, k);
 			ep2_mul_sim_basic(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim_basic(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim_basic(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim_basic(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -959,31 +918,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim_trick(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul(q, p, k);
 			ep2_mul_sim_trick(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim_trick(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim_trick(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim_trick(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -993,31 +954,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim_inter(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul(q, p, k);
 			ep2_mul_sim_inter(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim_inter(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim_inter(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim_inter(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -1027,31 +990,33 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim_joint(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul(q, p, k);
 			ep2_mul_sim_joint(r, p, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim_joint(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim_joint(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			ep2_norm(q, q);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim_joint(r, p, k, q, l);
 			ep2_mul(p, p, k);
 			ep2_mul(q, q, l);
 			ep2_add(q, q, p);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 #endif
 
@@ -1060,35 +1025,35 @@ static int simultaneous(void) {
 			bn_rand_mod(l, n);
 			ep2_mul(q, p, l);
 			ep2_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			ep2_mul_gen(q, k);
 			ep2_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			ep2_mul_sim_gen(r, k, q, l);
 			ep2_curve_get_gen(p);
 			ep2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			ep2_mul_sim_gen(r, k, q, l);
 			ep2_curve_get_gen(p);
 			ep2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			ep2_mul_sim_gen(r, k, q, l);
 			ep2_curve_get_gen(p);
 			ep2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(ep2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -1100,7 +1065,7 @@ static int simultaneous(void) {
 }
 
 static int compression(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a, b, c;
 
 	ep2_null(a);
@@ -1116,14 +1081,14 @@ static int compression(void) {
 			ep2_rand(a);
 			ep2_pck(b, a);
 			TEST_ASSERT(ep2_upk(c, b) == 1, end);
-			TEST_ASSERT(ep2_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -1132,7 +1097,7 @@ static int compression(void) {
 }
 
 static int hashing(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	bn_t n;
 	ep2_t p;
 	uint8_t msg[5];
@@ -1158,7 +1123,7 @@ static int hashing(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	ep2_free(p);
@@ -1166,7 +1131,7 @@ static int hashing(void) {
 }
 
 static int frobenius(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	ep2_t a, b, c;
 	bn_t d, n;
 
@@ -1188,35 +1153,18 @@ static int frobenius(void) {
 		TEST_BEGIN("frobenius and scalar multiplication are consistent") {
 			ep2_rand(a);
 			ep2_frb(b, a, 1);
-			d->used = FP_DIGS;
-			dv_copy(d->dp, fp_prime_get(), FP_DIGS);
+			d->used = RLC_FP_DIGS;
+			dv_copy(d->dp, fp_prime_get(), RLC_FP_DIGS);
 			bn_mod(d, d, n);
 			ep2_mul(c, a, d);
-			TEST_ASSERT(ep2_cmp(c, b) == CMP_EQ, end);
-		} TEST_END;
-
-		TEST_BEGIN("squared frobenius is consistent") {
-			ep2_rand(a);
-			ep2_frb(b, a, 1);
-			ep2_frb(b, b, 1);
-			ep2_frb(c, a, 2);
-			TEST_ASSERT(ep2_cmp(c, b) == CMP_EQ, end);
-		} TEST_END;
-
-		TEST_BEGIN("cubed frobenius is consistent") {
-			ep2_rand(a);
-			ep2_frb(b, a, 1);
-			ep2_frb(b, b, 1);
-			ep2_frb(b, b, 1);
-			ep2_frb(c, a, 3);
-			TEST_ASSERT(ep2_cmp(c, b) == CMP_EQ, end);
+			TEST_ASSERT(ep2_cmp(c, b) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	ep2_free(a);
 	ep2_free(b);
@@ -1227,14 +1175,14 @@ static int frobenius(void) {
 }
 
 int main(void) {
-	if (core_init() != STS_OK) {
+	if (core_init() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
 	util_banner("Tests for the EPX module", 0);
 
-	if (ep_param_set_any_pairf() == STS_ERR) {
+	if (ep_param_set_any_pairf() == RLC_ERR) {
 		THROW(ERR_NO_CURVE);
 		core_clean();
 		return 0;
@@ -1248,62 +1196,61 @@ int main(void) {
 
 	ep_param_print();
 
-	util_banner("Sextic twist:", 0);
 	util_banner("Utilities:", 1);
 
-	if (memory() != STS_OK) {
+	if (memory() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (util() != STS_OK) {
+	if (util() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
 	util_banner("Arithmetic:", 1);
 
-	if (addition() != STS_OK) {
+	if (addition() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (subtraction() != STS_OK) {
+	if (subtraction() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (doubling() != STS_OK) {
+	if (doubling() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (multiplication() != STS_OK) {
+	if (multiplication() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (fixed() != STS_OK) {
+	if (fixed() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (simultaneous() != STS_OK) {
+	if (simultaneous() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (compression() != STS_OK) {
+	if (compression() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (hashing() != STS_OK) {
+	if (hashing() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (frobenius() != STS_OK) {
+	if (frobenius() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
