@@ -1,23 +1,24 @@
 /*
  * RELIC is an Efficient LIbrary for Cryptography
- * Copyright (C) 2007-2017 RELIC Authors
+ * Copyright (C) 2007-2019 RELIC Authors
  *
  * This file is part of RELIC. RELIC is legal property of its developers,
  * whose names are not listed here. Please refer to the COPYRIGHT file
  * for contact information.
  *
- * RELIC is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * RELIC is free software; you can redistribute it and/or modify it under the
+ * terms of the version 2.1 (or later) of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; or version 2.0 of the Apache
+ * License as published by the Apache Software Foundation. See the LICENSE files
+ * for more details.
  *
- * RELIC is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
+ * RELIC is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+ * A PARTICULAR PURPOSE. See the LICENSE files for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with RELIC. If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public or the
+ * Apache License along with RELIC. If not, see <https://www.gnu.org/licenses/>
+ * or <https://www.apache.org/licenses/>.
  */
 
 /**
@@ -35,7 +36,7 @@
 
 static int memory1(void) {
 	err_t e;
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t a;
 
 	g1_null(a);
@@ -54,15 +55,15 @@ static int memory1(void) {
 		}
 	}
 	(void)a;
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	return code;
 }
 
 int util1(void) {
-	int l, code = STS_ERR;
+	int l, code = RLC_ERR;
 	g1_t a, b, c;
-	uint8_t bin[2 * PC_BYTES + 1];
+	uint8_t bin[2 * RLC_PC_BYTES + 1];
 
 	g1_null(a);
 	g1_null(b);
@@ -76,7 +77,7 @@ int util1(void) {
 		TEST_BEGIN("comparison is consistent") {
 			g1_rand(a);
 			g1_rand(b);
-			TEST_ASSERT(g1_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -85,32 +86,32 @@ int util1(void) {
 			g1_rand(b);
 			g1_rand(c);
 			/* Compare points in affine coordinates. */
-			if (g1_cmp(a, c) != CMP_EQ) {
+			if (g1_cmp(a, c) != RLC_EQ) {
 				g1_copy(c, a);
-				TEST_ASSERT(g1_cmp(c, a) == CMP_EQ, end);
+				TEST_ASSERT(g1_cmp(c, a) == RLC_EQ, end);
 			}
-			if (g1_cmp(b, c) != CMP_EQ) {
+			if (g1_cmp(b, c) != RLC_EQ) {
 				g1_copy(c, b);
-				TEST_ASSERT(g1_cmp(b, c) == CMP_EQ, end);
+				TEST_ASSERT(g1_cmp(b, c) == RLC_EQ, end);
 			}
 			/* Compare with one point in projective. */
 			g1_dbl(c, a);
 			g1_norm(c, c);
 			g1_dbl(a, a);
-			TEST_ASSERT(g1_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(g1_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(g1_cmp(a, c) == RLC_EQ, end);
 			/* Compare with two points in projective. */
 			g1_dbl(c, c);
 			g1_dbl(a, a);
-			TEST_ASSERT(g1_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(g1_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(g1_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 
 		TEST_BEGIN("inversion and comparison are consistent") {
 			g1_rand(a);
 			g1_neg(b, a);
-			TEST_ASSERT(g1_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -119,8 +120,8 @@ int util1(void) {
 		{
 			g1_rand(a);
 			g1_set_infty(c);
-			TEST_ASSERT(g1_cmp(a, c) != CMP_EQ, end);
-			TEST_ASSERT(g1_cmp(c, a) != CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(a, c) != RLC_EQ, end);
+			TEST_ASSERT(g1_cmp(c, a) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -136,19 +137,19 @@ int util1(void) {
 				l = g1_size_bin(a, j);
 				g1_write_bin(bin, l, a, j);
 				g1_read_bin(b, bin, l);
-				TEST_ASSERT(g1_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g1_cmp(a, b) == RLC_EQ, end);
 				g1_rand(a);
 				l = g1_size_bin(a, j);
 				g1_write_bin(bin, l, a, j);
 				g1_read_bin(b, bin, l);
-				TEST_ASSERT(g1_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g1_cmp(a, b) == RLC_EQ, end);
 				g1_rand(a);
 				g1_dbl(a, a);
 				l = g1_size_bin(a, j);
 				g1_norm(a, a);
 				g1_write_bin(bin, l, a, j);
 				g1_read_bin(b, bin, l);
-				TEST_ASSERT(g1_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g1_cmp(a, b) == RLC_EQ, end);
 			}
 		}
 		TEST_END;
@@ -157,7 +158,7 @@ int util1(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(a);
 	g1_free(b);
@@ -166,7 +167,7 @@ int util1(void) {
 }
 
 int addition1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 
 	g1_t a, b, c, d, e;
 
@@ -188,7 +189,7 @@ int addition1(void) {
 			g1_rand(b);
 			g1_add(d, a, b);
 			g1_add(e, b, a);
-			TEST_ASSERT(g1_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition is associative") {
@@ -199,16 +200,16 @@ int addition1(void) {
 			g1_add(d, d, c);
 			g1_add(e, b, c);
 			g1_add(e, e, a);
-			TEST_ASSERT(g1_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has identity") {
 			g1_rand(a);
 			g1_set_infty(d);
 			g1_add(e, a, d);
-			TEST_ASSERT(g1_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(e, a) == RLC_EQ, end);
 			g1_add(e, d, a);
-			TEST_ASSERT(g1_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(e, a) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has inverse") {
@@ -221,7 +222,7 @@ int addition1(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(a);
 	g1_free(b);
@@ -232,7 +233,7 @@ int addition1(void) {
 }
 
 int subtraction1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t a, b, c, d;
 
 	g1_null(a);
@@ -252,7 +253,7 @@ int subtraction1(void) {
 			g1_sub(c, a, b);
 			g1_sub(d, b, a);
 			g1_neg(d, d);
-			TEST_ASSERT(g1_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(c, d) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -260,7 +261,7 @@ int subtraction1(void) {
 			g1_rand(a);
 			g1_set_infty(c);
 			g1_sub(d, a, c);
-			TEST_ASSERT(g1_cmp(d, a) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(d, a) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -274,7 +275,7 @@ int subtraction1(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(a);
 	g1_free(b);
@@ -284,7 +285,7 @@ int subtraction1(void) {
 }
 
 int doubling1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t a, b, c;
 
 	g1_null(a);
@@ -300,13 +301,13 @@ int doubling1(void) {
 			g1_rand(a);
 			g1_add(b, a, a);
 			g1_dbl(c, a);
-			TEST_ASSERT(g1_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(a);
 	g1_free(b);
@@ -315,7 +316,7 @@ int doubling1(void) {
 }
 
 static int multiplication1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t p, q, r;
 	bn_t n, k;
 
@@ -346,15 +347,15 @@ static int multiplication1(void) {
 			TEST_ASSERT(g1_is_infty(r), end);
 			bn_set_dig(k, 1);
 			g1_mul_gen(r, k);
-			TEST_ASSERT(g1_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			g1_mul(q, p, k);
 			g1_mul_gen(r, k);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g1_mul_gen(r, k);
 			g1_neg(r, r);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("random element has the right order") {
@@ -367,7 +368,7 @@ static int multiplication1(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(p);
 	g1_free(q);
@@ -378,9 +379,9 @@ static int multiplication1(void) {
 }
 
 static int fixed1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t p, q, r;
-	g1_t t[RELIC_G1_TABLE];
+	g1_t t[RLC_G1_TABLE];
 	bn_t n, k;
 
 	bn_null(n);
@@ -389,7 +390,7 @@ static int fixed1(void) {
 	g1_null(q);
 	g1_null(r);
 
-	for (int i = 0; i < RELIC_G1_TABLE; i++) {
+	for (int i = 0; i < RLC_G1_TABLE; i++) {
 		g1_null(t[i]);
 	}
 
@@ -403,7 +404,7 @@ static int fixed1(void) {
 		g1_get_gen(p);
 		g1_get_ord(n);
 
-		for (int i = 0; i < RELIC_G1_TABLE; i++) {
+		for (int i = 0; i < RLC_G1_TABLE; i++) {
 			g1_new(t[i]);
 		}
 		TEST_BEGIN("fixed point multiplication is correct") {
@@ -414,18 +415,18 @@ static int fixed1(void) {
 			TEST_ASSERT(g1_is_infty(r), end);
 			bn_set_dig(k, 1);
 			g1_mul_fix(r, (const g1_t *)t, k);
-			TEST_ASSERT(g1_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			g1_mul(q, p, k);
 			g1_mul_fix(q, (const g1_t *)t, k);
 			g1_mul(r, p, k);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g1_mul_fix(r, (const g1_t *)t, k);
 			g1_neg(r, r);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_G1_TABLE; i++) {
+		for (int i = 0; i < RLC_G1_TABLE; i++) {
 			g1_free(t[i]);
 		}
 	}
@@ -433,7 +434,7 @@ static int fixed1(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(p);
 	g1_free(q);
@@ -444,7 +445,7 @@ static int fixed1(void) {
 }
 
 static int simultaneous1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t p, q, r;
 	bn_t n, k, l;
 
@@ -471,31 +472,31 @@ static int simultaneous1(void) {
 			bn_rand_mod(l, n);
 			g1_mul(q, p, l);
 			g1_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			g1_mul(q, p, k);
 			g1_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			g1_mul_sim(r, p, k, q, l);
 			g1_mul(p, p, k);
 			g1_mul(q, q, l);
 			g1_add(q, q, p);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g1_mul_sim(r, p, k, q, l);
 			g1_mul(p, p, k);
 			g1_mul(q, q, l);
 			g1_add(q, q, p);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			g1_mul_sim(r, p, k, q, l);
 			g1_mul(p, p, k);
 			g1_mul(q, q, l);
 			g1_add(q, q, p);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("simultaneous multiplication with generator is correct") {
@@ -503,35 +504,35 @@ static int simultaneous1(void) {
 			bn_rand_mod(l, n);
 			g1_mul(q, p, l);
 			g1_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			g1_mul_gen(q, k);
 			g1_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			g1_mul_sim_gen(r, k, q, l);
 			g1_get_gen(p);
 			g1_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g1_mul_sim_gen(r, k, q, l);
 			g1_get_gen(p);
 			g1_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			g1_mul_sim_gen(r, k, q, l);
 			g1_get_gen(p);
 			g1_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g1_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g1_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -542,8 +543,33 @@ static int simultaneous1(void) {
 	return code;
 }
 
+static int validity1(void) {
+	int code = RLC_ERR;
+	g1_t a;
+
+	g1_null(a);
+
+	TRY {
+		g1_new(a);
+
+		TEST_BEGIN("validity check is correct") {
+			g1_rand(a);
+			TEST_ASSERT(g1_is_valid(a) == 1, end);
+		}
+		TEST_END;
+
+	}
+	CATCH_ANY {
+		ERROR(end);
+	}
+	code = RLC_OK;
+  end:
+	g1_free(a);
+	return code;
+}
+
 static int hashing1(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g1_t a;
 	bn_t n;
 	uint8_t msg[5];
@@ -560,8 +586,7 @@ static int hashing1(void) {
 		TEST_BEGIN("point hashing is correct") {
 			rand_bytes(msg, sizeof(msg));
 			g1_map(a, msg, sizeof(msg));
-			g1_mul(a, a, n);
-			TEST_ASSERT(g1_is_infty(a) == 1, end);
+			TEST_ASSERT(g1_is_valid(a) == 1, end);
 		}
 		TEST_END;
 
@@ -569,7 +594,7 @@ static int hashing1(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g1_free(a);
 	bn_free(n);
@@ -578,7 +603,7 @@ static int hashing1(void) {
 
 static int memory2(void) {
 	err_t e;
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t a;
 
 	g2_null(a);
@@ -597,15 +622,15 @@ static int memory2(void) {
 		}
 	}
 	(void)a;
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	return code;
 }
 
 int util2(void) {
-	int l, code = STS_ERR;
+	int l, code = RLC_ERR;
 	g2_t a, b, c;
-	uint8_t bin[4 * PC_BYTES + 1];
+	uint8_t bin[4 * RLC_PC_BYTES + 1];
 
 	g2_null(a);
 	g2_null(b);
@@ -619,7 +644,7 @@ int util2(void) {
 		TEST_BEGIN("comparison is consistent") {
 			g2_rand(a);
 			g2_rand(b);
-			TEST_ASSERT(g2_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -628,32 +653,32 @@ int util2(void) {
 			g2_rand(b);
 			g2_rand(c);
 			/* Compare points in affine coordinates. */
-			if (g2_cmp(a, c) != CMP_EQ) {
+			if (g2_cmp(a, c) != RLC_EQ) {
 				g2_copy(c, a);
-				TEST_ASSERT(g2_cmp(c, a) == CMP_EQ, end);
+				TEST_ASSERT(g2_cmp(c, a) == RLC_EQ, end);
 			}
-			if (g2_cmp(b, c) != CMP_EQ) {
+			if (g2_cmp(b, c) != RLC_EQ) {
 				g2_copy(c, b);
-				TEST_ASSERT(g2_cmp(b, c) == CMP_EQ, end);
+				TEST_ASSERT(g2_cmp(b, c) == RLC_EQ, end);
 			}
 			/* Compare with one point in projective. */
 			g2_dbl(c, a);
 			g2_norm(c, c);
 			g2_dbl(a, a);
-			TEST_ASSERT(g2_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(g2_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(g2_cmp(a, c) == RLC_EQ, end);
 			/* Compare with two points in projective. */
 			g2_dbl(c, c);
 			g2_dbl(a, a);
-			TEST_ASSERT(g2_cmp(c, a) == CMP_EQ, end);
-			TEST_ASSERT(g2_cmp(a, c) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(c, a) == RLC_EQ, end);
+			TEST_ASSERT(g2_cmp(a, c) == RLC_EQ, end);
 		}
 		TEST_END;
 
 		TEST_BEGIN("negation and comparison are consistent") {
 			g2_rand(a);
 			g2_neg(b, a);
-			TEST_ASSERT(g2_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -662,8 +687,8 @@ int util2(void) {
 		{
 			g2_rand(a);
 			g2_set_infty(c);
-			TEST_ASSERT(g2_cmp(a, c) != CMP_EQ, end);
-			TEST_ASSERT(g2_cmp(c, a) != CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(a, c) != RLC_EQ, end);
+			TEST_ASSERT(g2_cmp(c, a) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -679,19 +704,19 @@ int util2(void) {
 				l = g2_size_bin(a, j);
 				g2_write_bin(bin, l, a, j);
 				g2_read_bin(b, bin, l);
-				TEST_ASSERT(g2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g2_cmp(a, b) == RLC_EQ, end);
 				g2_rand(a);
 				l = g2_size_bin(a, j);
 				g2_write_bin(bin, l, a, j);
 				g2_read_bin(b, bin, l);
-				TEST_ASSERT(g2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g2_cmp(a, b) == RLC_EQ, end);
 				g2_rand(a);
 				g2_dbl(a, a);
 				l = g2_size_bin(a, j);
 				g2_norm(a, a);
 				g2_write_bin(bin, l, a, j);
 				g2_read_bin(b, bin, l);
-				TEST_ASSERT(g2_cmp(a, b) == CMP_EQ, end);
+				TEST_ASSERT(g2_cmp(a, b) == RLC_EQ, end);
 			}
 		}
 		TEST_END;
@@ -700,7 +725,7 @@ int util2(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(a);
 	g2_free(b);
@@ -709,7 +734,7 @@ int util2(void) {
 }
 
 int addition2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 
 	g2_t a, b, c, d, e;
 
@@ -731,7 +756,7 @@ int addition2(void) {
 			g2_rand(b);
 			g2_add(d, a, b);
 			g2_add(e, b, a);
-			TEST_ASSERT(g2_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition is associative") {
@@ -742,16 +767,16 @@ int addition2(void) {
 			g2_add(d, d, c);
 			g2_add(e, b, c);
 			g2_add(e, e, a);
-			TEST_ASSERT(g2_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has identity") {
 			g2_rand(a);
 			g2_set_infty(d);
 			g2_add(e, a, d);
-			TEST_ASSERT(g2_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(e, a) == RLC_EQ, end);
 			g2_add(e, d, a);
-			TEST_ASSERT(g2_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(e, a) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("point addition has inverse") {
@@ -764,7 +789,7 @@ int addition2(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(a);
 	g2_free(b);
@@ -775,7 +800,7 @@ int addition2(void) {
 }
 
 int subtraction2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t a, b, c, d;
 
 	g2_null(a);
@@ -795,7 +820,7 @@ int subtraction2(void) {
 			g2_sub(c, a, b);
 			g2_sub(d, b, a);
 			g2_neg(d, d);
-			TEST_ASSERT(g2_cmp(c, d) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(c, d) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -803,7 +828,7 @@ int subtraction2(void) {
 			g2_rand(a);
 			g2_set_infty(c);
 			g2_sub(d, a, c);
-			TEST_ASSERT(g2_cmp(d, a) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(d, a) == RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -817,7 +842,7 @@ int subtraction2(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(a);
 	g2_free(b);
@@ -827,7 +852,7 @@ int subtraction2(void) {
 }
 
 int doubling2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t a, b, c;
 
 	g2_null(a);
@@ -843,13 +868,13 @@ int doubling2(void) {
 			g2_rand(a);
 			g2_add(b, a, a);
 			g2_dbl(c, a);
-			TEST_ASSERT(g2_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(a);
 	g2_free(b);
@@ -858,7 +883,7 @@ int doubling2(void) {
 }
 
 static int multiplication2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t p, q, r;
 	bn_t n, k;
 
@@ -889,15 +914,15 @@ static int multiplication2(void) {
 			TEST_ASSERT(g2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			g2_mul_gen(r, k);
-			TEST_ASSERT(g2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			g2_mul(q, p, k);
 			g2_mul_gen(r, k);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g2_mul_gen(r, k);
 			g2_neg(r, r);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("random element has the right order") {
@@ -910,7 +935,7 @@ static int multiplication2(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(p);
 	g2_free(q);
@@ -921,9 +946,9 @@ static int multiplication2(void) {
 }
 
 static int fixed2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t p, q, r;
-	g2_t t[RELIC_G2_TABLE];
+	g2_t t[RLC_G2_TABLE];
 	bn_t n, k;
 
 	bn_null(n);
@@ -932,7 +957,7 @@ static int fixed2(void) {
 	g2_null(q);
 	g2_null(r);
 
-	for (int i = 0; i < RELIC_G2_TABLE; i++) {
+	for (int i = 0; i < RLC_G2_TABLE; i++) {
 		g2_null(t[i]);
 	}
 
@@ -946,7 +971,7 @@ static int fixed2(void) {
 		g2_get_gen(p);
 		g2_get_ord(n);
 
-		for (int i = 0; i < RELIC_G2_TABLE; i++) {
+		for (int i = 0; i < RLC_G2_TABLE; i++) {
 			g2_new(t[i]);
 		}
 		TEST_BEGIN("fixed point multiplication is correct") {
@@ -957,18 +982,18 @@ static int fixed2(void) {
 			TEST_ASSERT(g2_is_infty(r), end);
 			bn_set_dig(k, 1);
 			g2_mul_fix(r, t, k);
-			TEST_ASSERT(g2_cmp(p, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(p, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			g2_mul(q, p, k);
 			g2_mul_fix(q, t, k);
 			g2_mul(r, p, k);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g2_mul_fix(r, t, k);
 			g2_neg(r, r);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
-		for (int i = 0; i < RELIC_G2_TABLE; i++) {
+		for (int i = 0; i < RLC_G2_TABLE; i++) {
 			g2_free(t[i]);
 		}
 	}
@@ -976,7 +1001,7 @@ static int fixed2(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(p);
 	g2_free(q);
@@ -987,7 +1012,7 @@ static int fixed2(void) {
 }
 
 static int simultaneous2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t p, q, r;
 	bn_t n, k, l;
 
@@ -1014,31 +1039,31 @@ static int simultaneous2(void) {
 			bn_rand_mod(l, n);
 			g2_mul(q, p, l);
 			g2_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			g2_mul(q, p, k);
 			g2_mul_sim(r, p, k, p, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			g2_mul_sim(r, p, k, q, l);
 			g2_mul(p, p, k);
 			g2_mul(q, q, l);
 			g2_add(q, q, p);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g2_mul_sim(r, p, k, q, l);
 			g2_mul(p, p, k);
 			g2_mul(q, q, l);
 			g2_add(q, q, p);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			g2_mul_sim(r, p, k, q, l);
 			g2_mul(p, p, k);
 			g2_mul(q, q, l);
 			g2_add(q, q, p);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("simultaneous multiplication with generator is correct") {
@@ -1046,35 +1071,35 @@ static int simultaneous2(void) {
 			bn_rand_mod(l, n);
 			g2_mul(q, p, l);
 			g2_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_zero(l);
 			g2_mul_gen(q, k);
 			g2_mul_sim_gen(r, k, p, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_rand_mod(k, n);
 			bn_rand_mod(l, n);
 			g2_mul_sim_gen(r, k, q, l);
 			g2_get_gen(p);
 			g2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(k, k);
 			g2_mul_sim_gen(r, k, q, l);
 			g2_get_gen(p);
 			g2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 			bn_neg(l, l);
 			g2_mul_sim_gen(r, k, q, l);
 			g2_get_gen(p);
 			g2_mul_sim(q, p, k, q, l);
-			TEST_ASSERT(g2_cmp(q, r) == CMP_EQ, end);
+			TEST_ASSERT(g2_cmp(q, r) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	bn_free(n);
 	bn_free(k);
@@ -1085,8 +1110,33 @@ static int simultaneous2(void) {
 	return code;
 }
 
+static int validity2(void) {
+	int code = RLC_ERR;
+	g2_t a;
+
+	g2_null(a);
+
+	TRY {
+		g2_new(a);
+
+		TEST_BEGIN("validity check is correct") {
+			g2_rand(a);
+			TEST_ASSERT(g2_is_valid(a) == 1, end);
+		}
+		TEST_END;
+
+	}
+	CATCH_ANY {
+		ERROR(end);
+	}
+	code = RLC_OK;
+  end:
+	g2_free(a);
+	return code;
+}
+
 static int hashing2(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	g2_t a;
 	bn_t n;
 	uint8_t msg[5];
@@ -1103,8 +1153,7 @@ static int hashing2(void) {
 		TEST_BEGIN("point hashing is correct") {
 			rand_bytes(msg, sizeof(msg));
 			g2_map(a, msg, sizeof(msg), 1);
-			g2_mul(a, a, n);
-			TEST_ASSERT(g2_is_infty(a) == 1, end);
+			TEST_ASSERT(g2_is_valid(a) == 1, end);
 		}
 		TEST_END;
 
@@ -1112,7 +1161,7 @@ static int hashing2(void) {
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	g2_free(a);
 	bn_free(n);
@@ -1121,7 +1170,7 @@ static int hashing2(void) {
 
 static int memory(void) {
 	err_t e;
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	gt_t a;
 
 	gt_null(a);
@@ -1140,13 +1189,13 @@ static int memory(void) {
 		}
 	}
 	(void)a;
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	return code;
 }
 
 int util(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	gt_t a, b, c;
 
 	gt_null(a);
@@ -1161,7 +1210,7 @@ int util(void) {
 		TEST_BEGIN("comparison is consistent") {
 			gt_rand(a);
 			gt_rand(b);
-			TEST_ASSERT(gt_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -1169,13 +1218,13 @@ int util(void) {
 			gt_rand(a);
 			gt_rand(b);
 			gt_rand(c);
-			if (gt_cmp(a, c) != CMP_EQ) {
+			if (gt_cmp(a, c) != RLC_EQ) {
 				gt_copy(c, a);
-				TEST_ASSERT(gt_cmp(c, a) == CMP_EQ, end);
+				TEST_ASSERT(gt_cmp(c, a) == RLC_EQ, end);
 			}
-			if (gt_cmp(b, c) != CMP_EQ) {
+			if (gt_cmp(b, c) != RLC_EQ) {
 				gt_copy(c, b);
-				TEST_ASSERT(gt_cmp(b, c) == CMP_EQ, end);
+				TEST_ASSERT(gt_cmp(b, c) == RLC_EQ, end);
 			}
 		}
 		TEST_END;
@@ -1183,7 +1232,7 @@ int util(void) {
 		TEST_BEGIN("inversion and comparison are consistent") {
 			gt_rand(a);
 			gt_inv(b, a);
-			TEST_ASSERT(gt_cmp(a, b) != CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(a, b) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -1192,8 +1241,8 @@ int util(void) {
 		{
 			gt_rand(a);
 			gt_set_unity(c);
-			TEST_ASSERT(gt_cmp(a, c) != CMP_EQ, end);
-			TEST_ASSERT(gt_cmp(c, a) != CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(a, c) != RLC_EQ, end);
+			TEST_ASSERT(gt_cmp(c, a) != RLC_EQ, end);
 		}
 		TEST_END;
 
@@ -1207,7 +1256,7 @@ int util(void) {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(a);
 	gt_free(b);
@@ -1216,7 +1265,7 @@ int util(void) {
 }
 
 int multiplication(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 
 	gt_t a, b, c, d, e;
 
@@ -1238,7 +1287,7 @@ int multiplication(void) {
 			gt_rand(b);
 			gt_mul(d, a, b);
 			gt_mul(e, b, a);
-			TEST_ASSERT(gt_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("multiplication is associative") {
@@ -1249,22 +1298,22 @@ int multiplication(void) {
 			gt_mul(d, d, c);
 			gt_mul(e, b, c);
 			gt_mul(e, e, a);
-			TEST_ASSERT(gt_cmp(d, e) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(d, e) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("multiplication has identity") {
 			gt_rand(a);
 			gt_set_unity(d);
 			gt_mul(e, a, d);
-			TEST_ASSERT(gt_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e, a) == RLC_EQ, end);
 			gt_mul(e, d, a);
-			TEST_ASSERT(gt_cmp(e, a) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e, a) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(a);
 	gt_free(b);
@@ -1275,7 +1324,7 @@ int multiplication(void) {
 }
 
 int squaring(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	gt_t a, b, c;
 
 	gt_null(a);
@@ -1291,13 +1340,13 @@ int squaring(void) {
 			gt_rand(a);
 			gt_mul(b, a, a);
 			gt_sqr(c, a);
-			TEST_ASSERT(gt_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(a);
 	gt_free(b);
@@ -1306,7 +1355,7 @@ int squaring(void) {
 }
 
 static int inversion(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	gt_t a, b, c;
 
 	TRY {
@@ -1319,14 +1368,14 @@ static int inversion(void) {
 			gt_inv(b, a);
 			gt_mul(c, a, b);
 			gt_set_unity(b);
-			TEST_ASSERT(gt_cmp(c, b) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(c, b) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(a);
 	gt_free(b);
@@ -1335,7 +1384,7 @@ static int inversion(void) {
 }
 
 int exponentiation(void) {
-	int code = STS_ERR;
+	int code = RLC_ERR;
 	gt_t a, b, c;
 	bn_t n, d;
 
@@ -1365,10 +1414,10 @@ int exponentiation(void) {
 			TEST_ASSERT(gt_is_unity(c), end);
 			bn_set_dig(d, 1);
 			gt_exp(c, a, d);
-			TEST_ASSERT(gt_cmp(c, a) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(c, a) == RLC_EQ, end);
 			bn_add_dig(d, n, 1);
 			gt_exp(c, a, d);
-			TEST_ASSERT(gt_cmp(c, a) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(c, a) == RLC_EQ, end);
 			gt_exp(c, a, n);
 			TEST_ASSERT(gt_is_unity(c), end);
 			bn_rand_mod(d, n);
@@ -1376,14 +1425,14 @@ int exponentiation(void) {
 			bn_neg(d, d);
 			gt_exp(c, a, d);
 			gt_inv(c, c);
-			TEST_ASSERT(gt_cmp(b, c) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(b, c) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(a);
 	gt_free(b);
@@ -1393,8 +1442,33 @@ int exponentiation(void) {
 	return code;
 }
 
+static int validity(void) {
+	int code = RLC_ERR;
+	gt_t a;
+
+	gt_null(a);
+
+	TRY {
+		gt_new(a);
+
+		TEST_BEGIN("validity check is correct") {
+			gt_rand(a);
+			TEST_ASSERT(gt_is_valid(a) == 1, end);
+		}
+		TEST_END;
+
+	}
+	CATCH_ANY {
+		ERROR(end);
+	}
+	code = RLC_OK;
+  end:
+	gt_free(a);
+	return code;
+}
+
 static int pairing(void) {
-	int j, code = STS_ERR;
+	int j, code = RLC_ERR;
 	gt_t e1, e2;
 	g1_t p[2];
 	g2_t q[2], r;
@@ -1426,14 +1500,14 @@ static int pairing(void) {
 			g1_rand(p[0]);
 			g2_rand(r);
 			pc_map(e1, p[0], r);
-			TEST_ASSERT(gt_cmp_dig(e1, 1) != CMP_EQ, end);
+			TEST_ASSERT(gt_cmp_dig(e1, 1) != RLC_EQ, end);
 			g1_set_infty(p[0]);
 			pc_map(e1, p[0], r);
-			TEST_ASSERT(gt_cmp_dig(e1, 1) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp_dig(e1, 1) == RLC_EQ, end);
 			g1_rand(p[0]);
 			g2_set_infty(r);
 			pc_map(e1, p[0], r);
-			TEST_ASSERT(gt_cmp_dig(e1, 1) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp_dig(e1, 1) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("pairing is bilinear") {
@@ -1444,18 +1518,18 @@ static int pairing(void) {
 			pc_map(e1, p[0], r);
 			pc_map(e2, p[0], q[0]);
 			gt_exp(e2, e2, k);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 			g1_mul(p[0], p[0], k);
 			pc_map(e2, p[0], q[0]);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 			g1_dbl(p[0], p[0]);
 			pc_map(e2, p[0], q[0]);
 			gt_sqr(e1, e1);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 			g2_dbl(q[0], q[0]);
 			pc_map(e2, p[0], q[0]);
 			gt_sqr(e1, e1);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 		} TEST_END;
 
 		TEST_BEGIN("multi-pairing is correct") {
@@ -1465,14 +1539,14 @@ static int pairing(void) {
 			g1_rand(p[1 - (i % 2)]);
 			g2_set_infty(q[1 - (i % 2)]);
 			pc_map_sim(e2, p, q, 2);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 			g1_set_infty(p[1 - (i % 2)]);
 			g2_rand(q[1 - (i % 2)]);
 			pc_map_sim(e2, p, q, 2);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 			g2_set_infty(q[i % 2]);
 			pc_map_sim(e2, p, q, 2);
-			TEST_ASSERT(gt_cmp_dig(e2, 1) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp_dig(e2, 1) == RLC_EQ, end);
 			g1_rand(p[0]);
 			g2_rand(q[0]);
 			pc_map(e1, p[0], q[0]);
@@ -1481,14 +1555,14 @@ static int pairing(void) {
 			pc_map(e2, p[1], q[1]);
 			gt_mul(e1, e1, e2);
 			pc_map_sim(e2, p, q, 2);
-			TEST_ASSERT(gt_cmp(e1, e2) == CMP_EQ, end);
+			TEST_ASSERT(gt_cmp(e1, e2) == RLC_EQ, end);
 		} TEST_END;
 	}
 	CATCH_ANY {
 		util_print("FATAL ERROR!\n");
 		ERROR(end);
 	}
-	code = STS_OK;
+	code = RLC_OK;
   end:
 	gt_free(e1);
 	gt_free(e2);
@@ -1505,139 +1579,151 @@ static int pairing(void) {
 int test1(void) {
 	util_banner("Utilities:", 1);
 
-	if (memory1() != STS_OK) {
+	if (memory1() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (util1() != STS_OK) {
-		return STS_ERR;
+	if (util1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
 	util_banner("Arithmetic:", 1);
 
-	if (addition1() != STS_OK) {
-		return STS_ERR;
+	if (addition1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (subtraction1() != STS_OK) {
-		return STS_ERR;
+	if (subtraction1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (doubling1() != STS_OK) {
-		return STS_ERR;
+	if (doubling1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (multiplication1() != STS_OK) {
-		return STS_ERR;
+	if (multiplication1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (fixed1() != STS_OK) {
-		return STS_ERR;
+	if (fixed1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (simultaneous1() != STS_OK) {
-		return STS_ERR;
+	if (simultaneous1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (hashing1() != STS_OK) {
-		return STS_ERR;
+	if (validity1() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	return STS_OK;
+	if (hashing1() != RLC_OK) {
+		return RLC_ERR;
+	}
+
+	return RLC_OK;
 }
 
 int test2(void) {
 	util_banner("Utilities:", 1);
 
-	if (memory2() != STS_OK) {
+	if (memory2() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (util2() != STS_OK) {
-		return STS_ERR;
+	if (util2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
 	util_banner("Arithmetic:", 1);
 
-	if (addition2() != STS_OK) {
-		return STS_ERR;
+	if (addition2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (subtraction2() != STS_OK) {
-		return STS_ERR;
+	if (subtraction2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (doubling2() != STS_OK) {
-		return STS_ERR;
+	if (doubling2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (multiplication2() != STS_OK) {
-		return STS_ERR;
+	if (multiplication2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (fixed2() != STS_OK) {
-		return STS_ERR;
+	if (fixed2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (simultaneous2() != STS_OK) {
-		return STS_ERR;
+	if (simultaneous2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (hashing2() != STS_OK) {
-		return STS_ERR;
+	if (validity2() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	return STS_OK;
+	if (hashing2() != RLC_OK) {
+		return RLC_ERR;
+	}
+
+	return RLC_OK;
 }
 
 int test(void) {
 	util_banner("Utilities:", 1);
 
-	if (memory() != STS_OK) {
+	if (memory() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
-	if (util() != STS_OK) {
-		return STS_ERR;
+	if (util() != RLC_OK) {
+		return RLC_ERR;
 	}
 
 	util_banner("Arithmetic:", 1);
 
-	if (multiplication() != STS_OK) {
-		return STS_ERR;
+	if (multiplication() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (squaring() != STS_OK) {
-		return STS_ERR;
+	if (squaring() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (inversion() != STS_OK) {
-		return STS_ERR;
+	if (inversion() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (exponentiation() != STS_OK) {
-		return STS_ERR;
+	if (exponentiation() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	if (pairing() != STS_OK) {
-		return STS_ERR;
+	if (validity() != RLC_OK) {
+		return RLC_ERR;
 	}
 
-	return STS_OK;
+	if (pairing() != RLC_OK) {
+		return RLC_ERR;
+	}
+
+	return RLC_OK;
 }
 
 int main(void) {
-	if (core_init() != STS_OK) {
+	if (core_init() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
 	util_banner("Tests for the PC module:", 0);
 
-	if (pc_param_set_any() != STS_OK) {
+	if (pc_param_set_any() != RLC_OK) {
 		THROW(ERR_NO_CURVE);
 		core_clean();
 		return 0;
@@ -1646,19 +1732,19 @@ int main(void) {
 	pc_param_print();
 
 	util_banner("Group G_1:", 0);
-	if (test1() != STS_OK) {
+	if (test1() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
 	util_banner("Group G_2:", 0);
-	if (test2() != STS_OK) {
+	if (test2() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
 
 	util_banner("Group G_T:", 0);
-	if (test() != STS_OK) {
+	if (test() != RLC_OK) {
 		core_clean();
 		return 1;
 	}
